@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Displayer.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -38,6 +38,7 @@ Displayer::Displayer() {
 	m_zoomoutbtn = MAIN->getWidget("button:main.zoomout");
 	m_zoomonebtn = MAIN->getWidget("button:main.zoomnormsize");
 	m_zoomfitbtn = MAIN->getWidget("button:main.zoomfit");
+	m_rotimage = MAIN->getWidget("image:display.rotate.mode");
 	m_rotspin = MAIN->getWidget("spin:display.rotate");
 	m_pagespin = MAIN->getWidget("spin:display.page");
 	m_resspin = MAIN->getWidget("spin:display.resolution");
@@ -45,6 +46,8 @@ Displayer::Displayer() {
 	m_conspin = MAIN->getWidget("spin:display.contrast");
 	m_invcheck = MAIN->getWidget("check:display.invert");
 
+	m_rotateMode = RotateMode::AllPages;
+
 #if GTKMM_CHECK_VERSION(3,12,0)
 	m_pagespin->set_icon_from_pixbuf(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/page.png"));
 	m_resspin->set_icon_from_pixbuf(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/resolution.png"));
@@ -60,12 +63,14 @@ Displayer::Displayer() {
 	m_scrollwin->drag_dest_set({Gtk::TargetEntry("text/uri-list")}, Gtk::DEST_DEFAULT_MOTION | Gtk::DEST_DEFAULT_DROP, Gdk::ACTION_COPY | Gdk::ACTION_MOVE);
 	m_viewport->override_background_color(Gdk::RGBA("#a0a0a4"));
 
+	CONNECT(MAIN->getWidget("menuitem:display.rotate.current").as<Gtk::MenuItem>(), activate, [this] { setRotateMode(RotateMode::CurrentPage, "rotate_page.png"); });
+	CONNECT(MAIN->getWidget("menuitem:display.rotate.all").as<Gtk::MenuItem>(), activate, [this] { setRotateMode(RotateMode::AllPages, "rotate_pages.png"); });
 	m_connection_rotSpinChanged = CONNECT(m_rotspin, value_changed, [this] { setAngle(m_rotspin->get_value()); });
 	m_connection_pageSpinChanged = CONNECT(m_pagespin, value_changed, [this] { setCurrentPage(m_pagespin->get_value_as_int()); });
-	m_connection_briSpinChanged = CONNECT(m_brispin, value_changed, [this] { queueRenderImage(); });
-	m_connection_conSpinChanged = CONNECT(m_conspin, value_changed, [this] { queueRenderImage(); });
-	m_connection_resSpinChanged = CONNECT(m_resspin, value_changed, [this] { queueRenderImage(); });
-	m_connection_invcheckToggled = CONNECT(m_invcheck, toggled, [this] { queueRenderImage(); });
+	m_connection_briSpinChanged = CONNECT(m_brispin, value_changed, [this] { brightnessChanged(); });
+	m_connection_conSpinChanged = CONNECT(m_conspin, value_changed, [this] { contrastChanged(); });
+	m_connection_resSpinChanged = CONNECT(m_resspin, value_changed, [this] { resolutionChanged(); });
+	m_connection_invcheckToggled = CONNECT(m_invcheck, toggled, [this] { invertColorsChanged(); });
 	CONNECT(m_viewport, size_allocate, [this](Gdk::Rectangle&) {
 		resizeEvent();
 	});
@@ -144,6 +149,10 @@ bool Displayer::setCurrentPage(int page) {
 	}
 	Source* source = m_pageMap[page].first;
 	if(source != m_currentSource) {
+		sendScaleRequest({ScaleRequest::Abort});
+		sendScaleRequest({ScaleRequest::Quit});
+		m_scaleThread->join();
+		m_scaleThread = nullptr;
 		delete m_renderer;
 		std::string filename = source->file->get_path();
 #ifdef G_OS_WIN32
@@ -157,7 +166,6 @@ bool Displayer::setCurrentPage(int page) {
 			m_renderer = new ImageRenderer(filename);
 			if(source->resolution == -1) source->resolution = 100;
 		}
-		Utils::set_spin_blocked(m_rotspin, source->angle / M_PI * 180., m_connection_rotSpinChanged);
 		Utils::set_spin_blocked(m_brispin, source->brightness, m_connection_briSpinChanged);
 		Utils::set_spin_blocked(m_conspin, source->contrast, m_connection_conSpinChanged);
 		Utils::set_spin_blocked(m_resspin, source->resolution, m_connection_resSpinChanged);
@@ -165,7 +173,9 @@ bool Displayer::setCurrentPage(int page) {
 		m_invcheck->set_active(source->invert);
 		m_connection_invcheckToggled.block(false);
 		m_currentSource = source;
+		m_scaleThread = Glib::Threads::Thread::create(sigc::mem_fun(this, &Displayer::scaleThread));
 	}
+	Utils::set_spin_blocked(m_rotspin, source->angle[m_pageMap[page].second - 1] / M_PI * 180., m_connection_rotSpinChanged);
 	Utils::set_spin_blocked(m_pagespin, page, m_connection_pageSpinChanged);
 	return renderImage();
 }
@@ -175,6 +185,7 @@ bool Displayer::setSources(std::vector<Source*> sources) {
 		return true;
 	}
 	if(m_scaleThread) {
+		sendScaleRequest({ScaleRequest::Abort});
 		sendScaleRequest({ScaleRequest::Quit});
 		m_scaleThread->join();
 		m_scaleThread = nullptr;
@@ -213,22 +224,32 @@ bool Displayer::setSources(std::vector<Source*> sources) {
 
 	int page = 0;
 	for(Source* source : m_sources) {
+		DisplayRenderer* renderer;
 		std::string filename = source->file->get_path();
 #ifdef G_OS_WIN32
 		if(Glib::ustring(filename.substr(filename.length() - 4)).lowercase() == ".pdf") {
 #else
 		if(Utils::get_content_type(filename) == "application/pdf") {
 #endif
-			PDFRenderer r(filename);
-			for(int pdfPage = 1, nPdfPages = r.getNPages(); pdfPage <= nPdfPages; ++pdfPage) {
-				m_pageMap.insert(std::make_pair(++page, std::make_pair(source, pdfPage)));
-			}
+			renderer = new PDFRenderer(filename);
 		} else {
-			m_pageMap.insert(std::make_pair(++page, std::make_pair(source, 1)));
+			renderer = new ImageRenderer(filename);
 		}
+		source->angle.resize(renderer->getNPages(), 0.);
+		for(int iPage = 1, nPages = renderer->getNPages(); iPage <= nPages; ++iPage) {
+			m_pageMap.insert(std::make_pair(++page, std::make_pair(source, iPage)));
+		}
+		delete renderer;
+	}
+	if(page == 0) {
+		m_pageMap.clear();
+		m_sources.clear();
+		return false;
 	}
 
+	m_connection_pageSpinChanged.block();
 	m_pagespin->get_adjustment()->set_upper(page);
+	m_connection_pageSpinChanged.unblock();
 	m_pagespin->set_visible(page > 1);
 	m_viewport->get_window()->set_cursor(Gdk::Cursor::create(Gdk::TCROSS));
 	m_canvas->show();
@@ -331,13 +352,30 @@ void Displayer::setZoom(Zoom zoom) {
 	m_connection_zoomoneClicked.block(false);
 }
 
+void Displayer::setRotateMode(RotateMode mode, const std::string& iconName) {
+	m_rotateMode = mode;
+#if GTKMM_CHECK_VERSION(3,12,0)
+	m_rotimage->set(Gdk::Pixbuf::create_from_resource(Glib::ustring::compose("/org/gnome/gimagereader/%1", iconName)));
+#else
+	m_rotimage->set(Glib::wrap(gdk_pixbuf_new_from_resource(Glib::ustring::compose("/org/gnome/gimagereader/%1", iconName).c_str(), 0)));
+#endif
+}
+
 void Displayer::setAngle(double angle) {
 	if(m_image) {
 		angle = angle < 0 ? angle + 360. : angle >= 360 ? angle - 360 : angle,
 		Utils::set_spin_blocked(m_rotspin, angle, m_connection_rotSpinChanged);
 		angle *= M_PI / 180.;
-		double delta = angle - m_currentSource->angle;
-		m_currentSource->angle = angle;
+		int sourcePage = m_pageMap[getCurrentPage()].second;
+		double delta = angle - m_currentSource->angle[sourcePage - 1];
+		if(m_rotateMode == RotateMode::CurrentPage) {
+			m_currentSource->angle[sourcePage - 1] = angle;
+		} else if(delta != 0) {
+			for(const auto& keyval  : m_pageMap) {
+				auto pair = keyval.second;
+				pair.first->angle[pair.second - 1] += delta;
+			}
+		}
 		m_imageItem->setRotation(angle);
 		if(m_tool) {
 			m_tool->rotationChanged(delta);
@@ -350,6 +388,38 @@ void Displayer::setAngle(double angle) {
 	}
 }
 
+void Displayer::brightnessChanged() {
+	int brightness = m_brispin->get_value_as_int();
+	for(const auto& keyval  : m_pageMap) {
+		keyval.second.first->brightness = brightness;
+	}
+	queueRenderImage();
+}
+
+void Displayer::contrastChanged() {
+	int contrast = m_conspin->get_value_as_int();
+	for(const auto& keyval  : m_pageMap) {
+		keyval.second.first->contrast = contrast;
+	}
+	queueRenderImage();
+}
+
+void Displayer::resolutionChanged() {
+	int resolution = m_resspin->get_value_as_int();
+	for(const auto& keyval  : m_pageMap) {
+		keyval.second.first->resolution = resolution;
+	}
+	queueRenderImage();
+}
+
+void Displayer::invertColorsChanged() {
+	bool invert = m_invcheck->get_active();
+	for(const auto& keyval  : m_pageMap) {
+		keyval.second.first->invert = invert;
+	}
+	queueRenderImage();
+}
+
 void Displayer::setResolution(int resolution) {
 	Utils::set_spin_blocked(m_resspin, resolution, m_connection_resSpinChanged);
 	renderImage();
@@ -527,7 +597,7 @@ void Displayer::resortItems() {
 Geometry::Rectangle Displayer::getSceneBoundingRect() const {
 	int w = m_image->get_width();
 	int h = m_image->get_height();
-	Geometry::Rotation R(m_currentSource->angle);
+	Geometry::Rotation R(m_rotspin->get_value() / 180. * M_PI);
 	int width = std::abs(R(0, 0) * w) + std::abs(R(0, 1) * h);
 	int height = std::abs(R(1, 0) * w) + std::abs(R(1, 1) * h);
 	return Geometry::Rectangle(-0.5 * width, -0.5 * height, width, height);
@@ -547,7 +617,7 @@ Cairo::RefPtr<Cairo::ImageSurface> Displayer::getImage(const Geometry::Rectangle
 	ctx->set_source_rgba(1., 1., 1., 1.);
 	ctx->paint();
 	ctx->translate(-rect.x, -rect.y);
-	ctx->rotate(m_currentSource->angle);
+	ctx->rotate(m_rotspin->get_value() / 180. * M_PI);
 	ctx->translate(-0.5 * m_image->get_width(), -0.5 * m_image->get_height());
 	ctx->set_source(m_image, 0, 0);
 	ctx->paint();
@@ -571,6 +641,7 @@ void Displayer::scaleThread() {
 		ScaleRequest req = m_scaleRequests.front();
 		m_scaleRequests.pop();
 		if(req.type == ScaleRequest::Quit) {
+			m_connection_setScaledImage.disconnect();
 			break;
 		} else if(req.type == ScaleRequest::Scale) {
 			m_scaleMutex.unlock();
@@ -593,14 +664,14 @@ void Displayer::scaleThread() {
 			m_scaleMutex.unlock();
 
 			double scale = req.scale;
-			Glib::signal_idle().connect_once([this,image,scale] { setScaledImage(image, scale); });
+			m_connection_setScaledImage = Glib::signal_idle().connect([this,image,scale] { setScaledImage(image); return false; });
 			m_scaleMutex.lock();
 		}
 	};
 	m_scaleMutex.unlock();
 }
 
-void Displayer::setScaledImage(Cairo::RefPtr<Cairo::ImageSurface> image, double scale) {
+void Displayer::setScaledImage(Cairo::RefPtr<Cairo::ImageSurface> image) {
 	m_scaleMutex.lock();
 	if(!m_scaleRequests.empty() && m_scaleRequests.front().type == ScaleRequest::Abort) {
 		m_scaleRequests.pop();
