@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * DisplayerToolSelect.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -196,7 +196,7 @@ void DisplayerToolSelect::autodetectLayout(bool noDeskew) {
 	// If a somewhat large deskew angle is detected, automatically rotate image and redetect layout,
 	// unless we already attempted to rotate (to prevent endless loops)
 	avgDeskew = Utils::round(((avgDeskew/nDeskew)/M_PI * 180.) * 10.) / 10.;
-	if(std::abs(avgDeskew > .1) && !noDeskew) {
+	if(std::abs(avgDeskew) > .1 && !noDeskew) {
 		m_displayer->setAngle(m_displayer->getCurrentAngle() - avgDeskew);
 		autodetectLayout(true);
 	} else {
@@ -243,7 +243,11 @@ void NumberedDisplayerSelection::showContextMenu(GdkEventButton* event) {
 		}),
 		CONNECT(MAIN->getWidget("button:selectionmenu.save").as<Gtk::Button>(), clicked, [&]{
 			loop->quit();
-			selmenu->hide(); // Explicitly hide here to avoid conflicts with file dialog which pops up
+			// Explicitly hide (and wait for hide event to be processed) to avoid key/mouse-grab conflicts with file dialog which pops up
+			selmenu->hide();
+			while(Gtk::Main::events_pending()) {
+				Gtk::Main::iteration(false);
+			}
 			static_cast<DisplayerToolSelect*>(m_tool)->saveSelection(this);
 		}),
 		CONNECT(selmenu, button_press_event, [&](GdkEventButton* ev) {
