@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * MainWindow.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -48,6 +48,7 @@
 #include <sys/prctl.h>
 #include <sys/wait.h>
 #endif
+#include <tesseract/baseapi.h>
 
 
 #if ENABLE_VERSIONCHECK
@@ -121,6 +122,7 @@ MainWindow::MainWindow()
 	m_outputPaneToggleButton = getWidget("button:main.outputpane");
 	m_window->set_icon_name("gimagereader");
 	m_aboutdialog->set_version(PACKAGE_VERSION);
+	getWidget("label:about.tesseractver").as<Gtk::Label>()->set_markup(Glib::ustring::compose("<small>%1 %2</small>", _("Using tesseract"), TESSERACT_VERSION_STR));
 
 	m_config = new Config;
 	m_acquirer = new Acquirer;
@@ -133,6 +135,7 @@ MainWindow::MainWindow()
 	m_idlegroup.push_back(getWidget("button:main.zoomnormsize"));
 	m_idlegroup.push_back(getWidget("button:main.zoomfit"));
 	m_idlegroup.push_back(getWidget("button:main.recognize"));
+	m_idlegroup.push_back(getWidget("menubutton:display.rotate.mode"));
 	m_idlegroup.push_back(getWidget("spin:display.rotate"));
 	m_idlegroup.push_back(getWidget("spin:display.page"));
 	m_idlegroup.push_back(getWidget("spin:display.brightness"));
@@ -141,6 +144,21 @@ MainWindow::MainWindow()
 	m_idlegroup.push_back(getWidget("button:main.autolayout"));
 	m_idlegroup.push_back(getWidget("menubutton:main.languages"));
 
+#if GTKMM_CHECK_VERSION(3,12,0)
+	getWidget("image:main.controls").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/controls.png"));
+	getWidget("image:display.rotate.mode").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/rotate_pages.png"));
+	getWidget("image:display.rotate.all").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/rotate_pages.png"));
+	getWidget("image:display.rotate.current").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/rotate_page.png"));
+	getWidget("image:main.autolayout").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/autolayout.png"));
+#else
+	getWidget("image:main.controls").as<Gtk::Image>()->set(Glib::wrap(gdk_pixbuf_new_from_resource("/org/gnome/gimagereader/controls.png", 0)));
+	getWidget("image:display.rotate.mode").as<Gtk::Image>()->set(Glib::wrap(gdk_pixbuf_new_from_resource("/org/gnome/gimagereader/rotate_pages.png", 0)));
+	getWidget("image:display.rotate.all").as<Gtk::Image>()->set(Glib::wrap(gdk_pixbuf_new_from_resource("/org/gnome/gimagereader/rotate_pages.png", 0)));
+	getWidget("image:display.rotate.current").as<Gtk::Image>()->set(Glib::wrap(gdk_pixbuf_new_from_resource("/org/gnome/gimagereader/rotate_page.png", 0)));
+	getWidget("image:main.autolayout").as<Gtk::Image>()->set(Glib::wrap(gdk_pixbuf_new_from_resource("/org/gnome/gimagereader/autolayout.png", 0)));
+#endif
+
+
 	CONNECT(m_window, delete_event, [this](GdkEventAny* ev) {
 		return closeEvent(ev);
 	});
@@ -177,6 +195,7 @@ MainWindow::MainWindow()
 		m_window->resize(geom[2], geom[3]);
 		m_window->move(geom[0], geom[1]);
 	}
+	getWidget("paned:output").as<Gtk::Paned>()->set_orientation(static_cast<Gtk::Orientation>(!getWidget("combo:config.settings.paneorient").as<Gtk::ComboBoxText>()->get_active_row_number()));
 #if ENABLE_VERSIONCHECK
 	if(m_config->getSetting<SwitchSetting>("updatecheck")->getValue()) {
 		m_newVerThread = Glib::Threads::Thread::create([this] { getNewestVersion(); });
@@ -425,8 +444,8 @@ void MainWindow::checkVersion(const Glib::ustring& newver) {
 
 	if(newver.compare(curver) > 0) {
 		addNotification(_("New version"), Glib::ustring::compose(_("gImageReader %1 is available"), newver), {
-			{_("Download"), []{ gtk_show_uri(Gdk::Screen::get_default()->gobj(), DOWNLOADURL, GDK_CURRENT_TIME, 0); return false; }},
-			{_("Changelog"), []{ gtk_show_uri(Gdk::Screen::get_default()->gobj(), CHANGELOGURL, GDK_CURRENT_TIME, 0); return false; }},
+			{_("Download"), [=]{ gtk_show_uri_on_window(static_cast<Gtk::Window*>(m_window)->gobj(), DOWNLOADURL, GDK_CURRENT_TIME, 0); return false; }},
+			{_("Changelog"), [=]{ gtk_show_uri_on_window(static_cast<Gtk::Window*>(m_window)->gobj(), CHANGELOGURL, GDK_CURRENT_TIME, 0); return false; }},
 			{_("Don't notify again"), [this]{ m_config->getSetting<SwitchSetting>("updatecheck")->setValue(false); return true; }}
 		});
 	}
