@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * OutputEditorHOCR.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@
 #include <podofo/doc/PdfPainter.h>
 #include <podofo/doc/PdfStreamedDocument.h>
 
+#include "CCITTFax4Encoder.hh"
 #include "DisplayerToolHOCR.hh"
 #include "FileDialogs.hh"
 #include "MainWindow.hh"
@@ -163,8 +164,20 @@ public:
 	void drawImage(const Geometry::Rectangle& bbox, const Cairo::RefPtr<Cairo::ImageSurface>& image, const PDFSettings& settings) override {
 		m_context->save();
 		m_context->move_to(bbox.x, bbox.y);
-		Cairo::RefPtr<Cairo::ImageSurface> img = Image::simulateFormat(image, settings.colorFormat);
-		m_context->set_source(img, bbox.x, bbox.y);
+		if(settings.compression == PDFSettings::CompressJpeg) {
+			Image img(image, settings.colorFormat, settings.conversionFlags);
+			uint8_t* buf = nullptr;
+			unsigned long bufLen = 0;
+			img.writeJpeg(settings.compressionQuality, buf, bufLen);
+			Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create("jpeg");
+			loader->write(buf, bufLen);
+			loader->close();
+			std::free(buf);
+			Gdk::Cairo::set_source_pixbuf(m_context, loader->get_pixbuf(), bbox.x, bbox.y);
+		} else {
+			Cairo::RefPtr<Cairo::ImageSurface> img = Image::simulateFormat(image, settings.colorFormat, settings.conversionFlags);
+			m_context->set_source(img, bbox.x, bbox.y);
+		}
 		m_context->paint();
 		m_context->restore();
 	}
@@ -209,8 +222,8 @@ public:
 
 class OutputEditorHOCR::PoDoFoPDFPainter : public OutputEditorHOCR::PDFPainter {
 public:
-	PoDoFoPDFPainter(PoDoFo::PdfDocument* document, PoDoFo::PdfPainter* painter, double scaleFactor, double imageScale)
-		: m_document(document), m_painter(painter), m_scaleFactor(scaleFactor), m_imageScale(imageScale) {
+	PoDoFoPDFPainter(PoDoFo::PdfDocument* document, PoDoFo::PdfPainter* painter, double scaleFactor)
+		: m_document(document), m_painter(painter), m_scaleFactor(scaleFactor) {
 		m_pageHeight = m_painter->GetPage()->GetPageSize().GetHeight();
 	}
 	void setFontSize(double pointSize) override {
@@ -221,18 +234,17 @@ public:
 		m_painter->DrawText(x * m_scaleFactor, m_pageHeight - y * m_scaleFactor, pdfString);
 	}
 	void drawImage(const Geometry::Rectangle& bbox, const Cairo::RefPtr<Cairo::ImageSurface>& image, const PDFSettings& settings) override {
+		Image img(image, settings.colorFormat, settings.conversionFlags);
 #if PODOFO_VERSION >= PODOFO_MAKE_VERSION(0,9,3)
 		PoDoFo::PdfImage pdfImage(m_document);
 #else
 		PoDoFo::PdfImageCompat pdfImage(m_document);
 #endif
-		Cairo::RefPtr<Cairo::ImageSurface> scaledImage = Image::scale(image, m_imageScale);
 		pdfImage.SetImageColorSpace(settings.colorFormat == Image::Format_RGB24 ? PoDoFo::ePdfColorSpace_DeviceRGB : PoDoFo::ePdfColorSpace_DeviceGray);
-		Image img(scaledImage, settings.colorFormat);
 		if(settings.compression == PDFSettings::CompressZip) {
 			PoDoFo::PdfMemoryInputStream is(reinterpret_cast<const char*>(img.data), img.bytesPerLine * img.height);
 			pdfImage.SetImageData(img.width, img.height, img.sampleSize, &is, {PoDoFo::ePdfFilter_FlateDecode});
-		} else {
+		} else if(settings.compression == PDFSettings::CompressJpeg) {
 			PoDoFo::PdfName dctFilterName(PoDoFo::PdfFilterFactory::FilterTypeToName(PoDoFo::ePdfFilter_DCTDecode));
 			pdfImage.GetObject()->GetDictionary().AddKey(PoDoFo::PdfName::KeyFilter, dctFilterName);
 			uint8_t* buf = nullptr;
@@ -241,8 +253,21 @@ public:
 			PoDoFo::PdfMemoryInputStream is(reinterpret_cast<const char*>(buf), bufLen);
 			pdfImage.SetImageDataRaw(img.width, img.height, img.sampleSize, &is);
 			std::free(buf);
+		} else if(settings.compression == PDFSettings::CompressFax4) {
+			PoDoFo::PdfName faxFilterName(PoDoFo::PdfFilterFactory::FilterTypeToName(PoDoFo::ePdfFilter_CCITTFaxDecode));
+			pdfImage.GetObject()->GetDictionary().AddKey(PoDoFo::PdfName::KeyFilter, faxFilterName);
+			PoDoFo::PdfDictionary decodeParams;
+			decodeParams.AddKey("Columns", PoDoFo::PdfObject(PoDoFo::pdf_int64(img.width)));
+			decodeParams.AddKey("Rows", PoDoFo::PdfObject(PoDoFo::pdf_int64(img.height)));
+			decodeParams.AddKey("K", PoDoFo::PdfObject(PoDoFo::pdf_int64(-1))); // K < 0 --- Pure two-dimensional encoding (Group 4)
+			pdfImage.GetObject()->GetDictionary().AddKey("DecodeParms", PoDoFo::PdfObject(decodeParams));
+			CCITTFax4Encoder encoder;
+			uint32_t encodedLen = 0;
+			uint8_t* encoded = encoder.encode(img.data, img.width, img.height, img.bytesPerLine, encodedLen);
+			PoDoFo::PdfMemoryInputStream is(reinterpret_cast<char*>(encoded), encodedLen);
+			pdfImage.SetImageDataRaw(img.width, img.height, img.sampleSize, &is);
 		}
-		m_painter->DrawImage(bbox.x * m_scaleFactor, m_pageHeight - (bbox.y + bbox.height) * m_scaleFactor, &pdfImage, m_scaleFactor / m_imageScale, m_scaleFactor / m_imageScale);
+		m_painter->DrawImage(bbox.x * m_scaleFactor, m_pageHeight - (bbox.y + bbox.height) * m_scaleFactor, &pdfImage, m_scaleFactor * bbox.width / double(image->get_width()), m_scaleFactor * bbox.height / double(image->get_height()));
 	}
 	double getAverageCharWidth() const override {
 		return m_painter->GetFont()->GetFontMetrics()->CharWidth(static_cast<unsigned char>('x')) / m_scaleFactor;
@@ -280,6 +305,8 @@ OutputEditorHOCR::OutputEditorHOCR(DisplayerToolHOCR* tool)
 	Gtk::TreeView_Private::_connect_auto_store_editable_signal_handler<bool>(m_itemView, itemViewCol->get_cells()[0], m_itemStoreCols.selected);
 	itemViewCol->pack_start(m_itemStoreCols.icon, false);
 	itemViewCol->pack_start(m_itemStoreCols.text, true);
+	itemViewCol->add_attribute(*itemViewCol->get_cells()[0], "visible", m_itemStoreCols.checkboxVisible);
+	itemViewCol->add_attribute(*itemViewCol->get_cells()[1], "visible", m_itemStoreCols.iconVisible);
 	Gtk::TreeView_Private::_connect_auto_store_editable_signal_handler<Glib::ustring>(m_itemView, itemViewCol->get_cells()[2], m_itemStoreCols.text);
 	m_itemView->append_column(*itemViewCol);
 	m_itemView->set_expander_column(*m_itemView->get_column(0));
@@ -288,6 +315,11 @@ OutputEditorHOCR::OutputEditorHOCR(DisplayerToolHOCR* tool)
 		itemViewCol->add_attribute(textRenderer->property_foreground(), m_itemStoreCols.textColor);
 		itemViewCol->add_attribute(textRenderer->property_editable(), m_itemStoreCols.editable);
 	}
+	Gtk::TreeIter rootItem = m_itemStore->append();
+	rootItem->set_value(m_itemStoreCols.text, Glib::ustring(_("Document")));
+	rootItem->set_value(m_itemStoreCols.checkboxVisible, false);
+	rootItem->set_value(m_itemStoreCols.iconVisible, false);
+	m_rootItem = m_itemStore->get_path(rootItem);
 
 	m_propView = m_builder("treeview:hocr.properties");
 	m_propStore = Gtk::TreeStore::create(m_propStoreCols);
@@ -337,17 +369,39 @@ OutputEditorHOCR::OutputEditorHOCR(DisplayerToolHOCR* tool)
 	imageFormatCombo->pack_start(m_formatComboCols.label);
 	imageFormatCombo->set_active(-1);
 
+	Gtk::ComboBox* ditheringCombo = m_builder("combo:pdfoptions.dithering");
+	Glib::RefPtr<Gtk::ListStore> ditheringComboModel = Gtk::ListStore::create(m_ditheringComboCols);
+	ditheringCombo->set_model(ditheringComboModel);
+	row = *(ditheringComboModel->append());
+	row[m_ditheringComboCols.conversionFlags] = Image::ThresholdDithering;
+	row[m_ditheringComboCols.label] = _("Threshold (closest color)");
+	row = *(ditheringComboModel->append());
+	row[m_ditheringComboCols.conversionFlags] = Image::DiffuseDithering;
+	row[m_ditheringComboCols.label] = _("Diffuse");
+	ditheringCombo->pack_start(m_ditheringComboCols.label);
+	ditheringCombo->set_active(-1);
+
 	Gtk::ComboBox* compressionCombo = m_builder("combo:pdfoptions.compression");
 	Glib::RefPtr<Gtk::ListStore> compressionModel = Gtk::ListStore::create(m_compressionComboCols);
 	compressionCombo->set_model(compressionModel);
 	row = *(compressionModel->append());
 	row[m_compressionComboCols.mode] = PDFSettings::CompressZip;
 	row[m_compressionComboCols.label] = _("Zip (lossless)");
+	row[m_compressionComboCols.sensitive] = true;
+	row = *(compressionModel->append());
+	row[m_compressionComboCols.mode] = PDFSettings::CompressFax4;
+	row[m_compressionComboCols.label] = _("CCITT Group 4 (lossless)");
+	row[m_compressionComboCols.sensitive] = true;
 	row = *(compressionModel->append());
 	row[m_compressionComboCols.mode] = PDFSettings::CompressJpeg;
 	row[m_compressionComboCols.label] = _("Jpeg (lossy)");
+	row[m_compressionComboCols.sensitive] = true;
 	compressionCombo->pack_start(m_compressionComboCols.label);
 	compressionCombo->set_active(-1);
+	Gtk::CellRendererText* compressionTextRenderer = dynamic_cast<Gtk::CellRendererText*>(compressionCombo->get_cells()[0]);
+	if(compressionTextRenderer) {
+		compressionCombo->add_attribute(compressionTextRenderer->property_sensitive(), m_compressionComboCols.sensitive);
+	}
 
 	CONNECT(openButton, clicked, [this] { open(); });
 	CONNECT(saveButton, clicked, [this] { save(); });
@@ -374,7 +428,9 @@ OutputEditorHOCR::OutputEditorHOCR(DisplayerToolHOCR* tool)
 
 	CONNECT(m_builder("combo:pdfoptions.mode").as<Gtk::ComboBox>(), changed, [this] { updatePreview(); });
 	CONNECT(imageFormatCombo, changed, [this] { imageFormatChanged(); updatePreview(); });
+	CONNECT(ditheringCombo, changed, [this] { updatePreview(); });
 	CONNECT(compressionCombo, changed, [this] { imageCompressionChanged(); });
+	CONNECT(m_builder("spin:pdfoptions.quality").as<Gtk::SpinButton>(), value_changed, [this] { updatePreview(); });
 	CONNECT(m_builder("fontbutton:pdfoptions").as<Gtk::FontButton>(), font_set, [this] { updatePreview(); });
 	CONNECT(m_builder("checkbox:pdfoptions.usedetectedfontsizes").as<Gtk::CheckButton>(), toggled, [this] { updatePreview(); });
 	CONNECTS(m_builder("checkbox:pdfoptions.usedetectedfontsizes").as<Gtk::CheckButton>(), toggled, [this](Gtk::CheckButton* button) {
@@ -397,6 +453,7 @@ OutputEditorHOCR::OutputEditorHOCR(DisplayerToolHOCR* tool)
 	MAIN->getConfig()->addSetting(new SpinSetting("pdfimagecompressionquality", m_builder("spin:pdfoptions.quality")));
 	MAIN->getConfig()->addSetting(new ComboSetting("pdfimagecompression", m_builder("combo:pdfoptions.compression")));
 	MAIN->getConfig()->addSetting(new ComboSetting("pdfimageformat", m_builder("combo:pdfoptions.imageformat")));
+	MAIN->getConfig()->addSetting(new ComboSetting("pdfimageconversionflags", m_builder("combo:pdfoptions.dithering")));
 	MAIN->getConfig()->addSetting(new SpinSetting("pdfimagedpi", m_builder("spin:pdfoptions.dpi")));
 	MAIN->getConfig()->addSetting(new FontSetting("pdffont", m_builder("fontbutton:pdfoptions")));
 	MAIN->getConfig()->addSetting(new SwitchSettingT<Gtk::CheckButton>("pdfusedetectedfontsizes", m_builder("checkbox:pdfoptions.usedetectedfontsizes")));
@@ -444,22 +501,34 @@ void OutputEditorHOCR::setFont() {
 }
 
 void OutputEditorHOCR::imageFormatChanged() {
-	Image::Format format = (*m_builder("combo:pdfoptions.imageformat").as<Gtk::ComboBox>()->get_active())[m_formatComboCols.format];
+	Gtk::ComboBox* formatCombo = m_builder("combo:pdfoptions.imageformat");
+	Gtk::ComboBox* compressionCombo = m_builder("combo:pdfoptions.compression");
+	Glib::RefPtr<Gtk::ListStore> compressionStore = Glib::RefPtr<Gtk::ListStore>::cast_static<Gtk::TreeModel>(compressionCombo->get_model());
+	Image::Format format = (*formatCombo->get_active())[m_formatComboCols.format];
 	if(format == Image::Format_Mono) {
-		m_builder("combo:pdfoptions.compression").as<Gtk::ComboBox>()->set_active(0);
-		m_builder("combo:pdfoptions.compression").as<Gtk::Widget>()->set_sensitive(false);
-		m_builder("label:pdfoptions.compression").as<Gtk::Widget>()->set_sensitive(false);
+		if((*compressionCombo->get_active())[m_compressionComboCols.mode] == PDFSettings::CompressJpeg) {
+			compressionCombo->set_active(PDFSettings::CompressZip);
+		}
+		(*compressionStore->children()[PDFSettings::CompressFax4])[m_compressionComboCols.sensitive] = true;
+		(*compressionStore->children()[PDFSettings::CompressJpeg])[m_compressionComboCols.sensitive] = false;
+		m_builder("label:pdfoptions.dithering")->set_sensitive(true);
+		m_builder("combo:pdfoptions.dithering")->set_sensitive(true);
 	} else {
-		m_builder("combo:pdfoptions.compression").as<Gtk::Widget>()->set_sensitive(true);
-		m_builder("label:pdfoptions.compression").as<Gtk::Widget>()->set_sensitive(true);
+		if((*compressionCombo->get_active())[m_compressionComboCols.mode] == PDFSettings::CompressFax4) {
+			compressionCombo->set_active(PDFSettings::CompressZip);
+		}
+		(*compressionStore->children()[PDFSettings::CompressFax4])[m_compressionComboCols.sensitive] = false;
+		(*compressionStore->children()[PDFSettings::CompressJpeg])[m_compressionComboCols.sensitive] = true;
+		m_builder("label:pdfoptions.dithering")->set_sensitive(false);
+		m_builder("combo:pdfoptions.dithering")->set_sensitive(false);
 	}
 }
 
 void OutputEditorHOCR::imageCompressionChanged() {
 	PDFSettings::Compression compression = (*m_builder("combo:pdfoptions.compression").as<Gtk::ComboBox>()->get_active())[m_compressionComboCols.mode];
-	bool zipCompression = compression == PDFSettings::CompressZip;
-	m_builder("spin:pdfoptions.quality").as<Gtk::Widget>()->set_sensitive(!zipCompression);
-	m_builder("label:pdfoptions.quality").as<Gtk::Widget>()->set_sensitive(!zipCompression);
+	bool jpegCompression = compression == PDFSettings::CompressJpeg;
+	m_builder("spin:pdfoptions.quality").as<Gtk::Widget>()->set_sensitive(jpegCompression);
+	m_builder("label:pdfoptions.quality").as<Gtk::Widget>()->set_sensitive(jpegCompression);
 }
 
 OutputEditorHOCR::ReadSessionData* OutputEditorHOCR::initRead(tesseract::TessBaseAPI &tess) {
@@ -522,7 +591,7 @@ void OutputEditorHOCR::addPage(xmlpp::Element* pageDiv, const Glib::ustring& fil
 	int x2 = std::atoi(matchInfo.fetch(3).c_str());
 	int y2 = std::atoi(matchInfo.fetch(4).c_str());
 
-	Gtk::TreeIter pageItem = m_itemStore->append();
+	Gtk::TreeIter pageItem = m_itemStore->append(m_itemStore->get_iter(m_rootItem)->children());
 	pageItem->set_value(m_itemStoreCols.text, Glib::ustring::compose("%1 [%2]", filename, page));
 	pageItem->set_value(m_itemStoreCols.id, getAttribute(pageDiv, "id"));
 	pageItem->set_value(m_itemStoreCols.bbox, Geometry::Rectangle(x1, y1, x2-x1, y2-y1));
@@ -535,6 +604,8 @@ void OutputEditorHOCR::addPage(xmlpp::Element* pageDiv, const Glib::ustring& fil
 	pageItem->set_value(m_itemStoreCols.selected, true);
 	pageItem->set_value(m_itemStoreCols.editable, false);
 	pageItem->set_value(m_itemStoreCols.textColor, Glib::ustring("#000"));
+	pageItem->set_value(m_itemStoreCols.checkboxVisible, true);
+	pageItem->set_value(m_itemStoreCols.iconVisible, true);
 
 	std::map<Glib::ustring,Glib::ustring> langCache;
 
@@ -585,11 +656,14 @@ void OutputEditorHOCR::addPage(xmlpp::Element* pageDiv, const Glib::ustring& fil
 			item->set_value(m_itemStoreCols.itemClass, Glib::ustring("ocr_graphic"));
 			item->set_value(m_itemStoreCols.textColor, Glib::ustring("#000"));
 			item->set_value(m_itemStoreCols.bbox, Geometry::Rectangle(x1, y1, x2-x1, y2-y1));
+			item->set_value(m_itemStoreCols.checkboxVisible, true);
+			item->set_value(m_itemStoreCols.iconVisible, true);
 		} else {
 			element->get_parent()->remove_child(element);
 		}
 	}
 	pageItem->set_value(m_itemStoreCols.source, getElementXML(pageDiv));
+	m_itemView->expand_to_path(Gtk::TreePath(pageItem));
 	m_itemView->expand_row(Gtk::TreePath(pageItem), true);
 	MAIN->setOutputPaneVisible(true);
 	m_modified = true;
@@ -681,6 +755,8 @@ bool OutputEditorHOCR::addChildItems(xmlpp::Element* element, Gtk::TreeIter pare
 						}
 					}
 					item->set_value(m_itemStoreCols.text, title);
+					item->set_value(m_itemStoreCols.checkboxVisible, true);
+					item->set_value(m_itemStoreCols.iconVisible, true);
 				} else {
 					m_itemStore->erase(item);
 				}
@@ -702,12 +778,13 @@ void OutputEditorHOCR::showItemProperties(Gtk::TreeIter item) {
 	m_currentElement = nullptr;
 	delete m_currentParser;
 	m_currentParser = nullptr;
-	if(!item) {
+	Gtk::TreeIter rootIter = m_itemStore->get_iter(m_rootItem);
+	if(!item || item == rootIter) {
 		return;
 	}
 	m_currentItem = m_itemStore->get_path(item);
 	Gtk::TreeIter parentItem = item;
-	while(parentItem->parent()) {
+	while(parentItem->parent() != rootIter) {
 		parentItem = parentItem->parent();
 	}
 	m_currentPageItem = m_itemStore->get_path(parentItem);
@@ -758,7 +835,7 @@ void OutputEditorHOCR::showItemProperties(Gtk::TreeIter item) {
 	}
 }
 
-bool OutputEditorHOCR::setCurrentSource(xmlpp::Element* pageElement, int* pageDpi) const {
+bool OutputEditorHOCR::setCurrentSource(xmlpp::Element* pageElement, int* pageDpi, int* overrideDpi) const {
 	Glib::MatchInfo matchInfo;
 	Glib::ustring titleAttr = getAttribute(pageElement, "title");
 	if(s_pageTitleRx->match(titleAttr, matchInfo)) {
@@ -769,6 +846,9 @@ bool OutputEditorHOCR::setCurrentSource(xmlpp::Element* pageElement, int* pageDp
 		if(pageDpi) {
 			*pageDpi = res;
 		}
+		if(overrideDpi) {
+			res = *overrideDpi;
+		}
 
 		MAIN->getSourceManager()->addSources({Gio::File::create_for_path(filename)});
 		while(Gtk::Main::events_pending()) {
@@ -966,6 +1046,8 @@ void OutputEditorHOCR::addGraphicRection(const Geometry::Rectangle &rect) {
 	item->set_value(m_itemStoreCols.itemClass, Glib::ustring("ocr_graphic"));
 	item->set_value(m_itemStoreCols.textColor, Glib::ustring("#000"));
 	item->set_value(m_itemStoreCols.bbox, rect);
+	item->set_value(m_itemStoreCols.checkboxVisible, true);
+	item->set_value(m_itemStoreCols.iconVisible, true);
 
 	m_itemView->get_selection()->unselect_all();
 	m_itemView->get_selection()->select(item);
@@ -1074,78 +1156,81 @@ void OutputEditorHOCR::showContextMenu(GdkEventButton* ev) {
 		return;
 	}
 	Glib::ustring itemClass = (*it)[m_itemStoreCols.itemClass];
-
+	Gtk::Menu menu;
 	if(itemClass == "ocr_page") {
-		// Context menu on page items with Remove option
-		Gtk::Menu menu;
-		Glib::RefPtr<Glib::MainLoop> loop = Glib::MainLoop::create();
 		Gtk::MenuItem* addGraphicItem = Gtk::manage(new Gtk::MenuItem(_("Add graphic region")));
 		menu.append(*addGraphicItem);
-		menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem));
-		Gtk::MenuItem* removeItem = Gtk::manage(new Gtk::MenuItem(_("Remove")));
-		menu.append(*removeItem);
-		CONNECT(removeItem, activate, [&] {
-			m_itemStore->erase(it);
-			m_connectionPropViewRowEdited.block(true);
-			m_propStore->clear();
-			m_connectionPropViewRowEdited.block(false);
-			m_builder("button:hocr.save")->set_sensitive(!m_itemStore->children().empty());
-			m_builder("button:hocr.export")->set_sensitive(!m_itemStore->children().empty());
-		});
 		CONNECT(addGraphicItem, activate, [this] {
 			m_tool->clearSelection();
 			m_tool->activateDrawSelection();
 		});
-		CONNECT(&menu, hide, [&] { loop->quit(); });
-		menu.show_all();
-		menu.popup(ev->button, ev->time);
-		loop->run();
-		return;
-	} else {
-		// Context menu on word items with spelling suggestions, if any
-		Gtk::Menu menu;
-		Glib::RefPtr<Glib::MainLoop> loop = Glib::MainLoop::create();
-		if(itemClass == "ocrx_word") {
-			Glib::ustring prefix, suffix, trimmed = trimWord((*it)[m_itemStoreCols.text], &prefix, &suffix);
-			for(const Glib::ustring& suggestion : m_spell.get_suggestions(trimmed)) {
-				Glib::ustring replacement = prefix + suggestion + suffix;
-				Gtk::MenuItem* item = Gtk::manage(new Gtk::MenuItem(replacement));
-				CONNECT(item, activate, [this, replacement, it] { (*it)[m_itemStoreCols.text] = replacement; });
-				menu.append(*item);
-			}
-			if(menu.get_children().empty()) {
-				Gtk::MenuItem* item = Gtk::manage(new Gtk::MenuItem(_("No suggestions")));
-				item->set_sensitive(false);
-				menu.append(*item);
-			}
-			if(!m_spell.check_word(trimWord((*it)[m_itemStoreCols.text]))) {
-				menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem));
-				Gtk::MenuItem* additem = Gtk::manage(new Gtk::MenuItem(_("Add to dictionary")));
-				CONNECT(additem, activate, [this, it] {
-					m_spell.add_to_dictionary((*it)[m_itemStoreCols.text]);
-					it->set_value(m_itemStoreCols.textColor, Glib::ustring("#000"));
-				});
-				menu.append(*additem);
-				Gtk::MenuItem* ignoreitem = Gtk::manage(new Gtk::MenuItem(_("Ignore word")));
-				CONNECT(ignoreitem, activate, [this, it] {
-					m_spell.ignore_word((*it)[m_itemStoreCols.text]);
-					it->set_value(m_itemStoreCols.textColor, Glib::ustring("#000"));
-				});
-				menu.append(*ignoreitem);
-			}
+	}
+	if(itemClass == "ocrx_word") {
+		Glib::ustring prefix, suffix, trimmed = trimWord((*it)[m_itemStoreCols.text], &prefix, &suffix);
+		for(const Glib::ustring& suggestion : m_spell.get_suggestions(trimmed)) {
+			Glib::ustring replacement = prefix + suggestion + suffix;
+			Gtk::MenuItem* item = Gtk::manage(new Gtk::MenuItem(replacement));
+			CONNECT(item, activate, [this, replacement, it] { (*it)[m_itemStoreCols.text] = replacement; });
+			menu.append(*item);
+		}
+		if(menu.get_children().empty()) {
+			Gtk::MenuItem* item = Gtk::manage(new Gtk::MenuItem(_("No suggestions")));
+			item->set_sensitive(false);
+			menu.append(*item);
+		}
+		if(!m_spell.check_word(trimWord((*it)[m_itemStoreCols.text]))) {
+			menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem));
+			Gtk::MenuItem* additem = Gtk::manage(new Gtk::MenuItem(_("Add to dictionary")));
+			CONNECT(additem, activate, [this, it] {
+				m_spell.add_to_dictionary((*it)[m_itemStoreCols.text]);
+				it->set_value(m_itemStoreCols.textColor, Glib::ustring("#000"));
+			});
+			menu.append(*additem);
+			Gtk::MenuItem* ignoreitem = Gtk::manage(new Gtk::MenuItem(_("Ignore word")));
+			CONNECT(ignoreitem, activate, [this, it] {
+				m_spell.ignore_word((*it)[m_itemStoreCols.text]);
+				it->set_value(m_itemStoreCols.textColor, Glib::ustring("#000"));
+			});
+			menu.append(*ignoreitem);
+		}
+	}
+	if(path != m_rootItem) {
+		if(!menu.get_children().empty()) {
 			menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem));
 		}
-		Gtk::MenuItem* removeItem = Gtk::manage(new Gtk::MenuItem(_("Remove")));
-		menu.append(*removeItem);
-		CONNECT(removeItem, activate, [this] { removeCurrentItem(); });
-
-		CONNECT(&menu, hide, [&] { loop->quit(); });
-		menu.show_all();
-		menu.popup(ev->button, ev->time);
-		loop->run();
-		return;
+		if(itemClass == "ocr_page") {
+			Gtk::MenuItem* removeItem = Gtk::manage(new Gtk::MenuItem(_("Remove")));
+			menu.append(*removeItem);
+			CONNECT(removeItem, activate, [&] {
+				m_itemStore->erase(it);
+				m_connectionPropViewRowEdited.block(true);
+				m_propStore->clear();
+				m_connectionPropViewRowEdited.block(false);
+				m_builder("button:hocr.save")->set_sensitive(!m_itemStore->get_iter(m_rootItem)->children().empty());
+				m_builder("button:hocr.export")->set_sensitive(!m_itemStore->get_iter(m_rootItem)->children().empty());
+			});
+		} else {
+			Gtk::MenuItem* removeItem = Gtk::manage(new Gtk::MenuItem(_("Remove")));
+			menu.append(*removeItem);
+			CONNECT(removeItem, activate, [this] { removeCurrentItem(); });
+		}
 	}
-	return;
+	if(itemClass != "ocrx_word" && itemClass != "ocr_graphic") {
+		if(!menu.get_children().empty()) {
+			menu.append(*Gtk::manage(new Gtk::SeparatorMenuItem));
+		}
+		Gtk::MenuItem* expandItem = Gtk::manage(new Gtk::MenuItem(_("Expand all")));
+		menu.append(*expandItem);
+		CONNECT(expandItem, activate, [this, path] { m_itemView->expand_row(path, true); });
+		Gtk::MenuItem* collapseItem = Gtk::manage(new Gtk::MenuItem(_("Collapse all")));
+		menu.append(*collapseItem);
+		CONNECT(collapseItem, activate, [this, path] { m_itemView->collapse_row(path); });
+	}
+	Glib::RefPtr<Glib::MainLoop> loop = Glib::MainLoop::create();
+	CONNECT(&menu, hide, [&] { loop->quit(); });
+	menu.show_all();
+	menu.popup(ev->button, ev->time);
+	loop->run();
 }
 
 void OutputEditorHOCR::checkCellEditable(const Glib::ustring& path, Gtk::CellRenderer* renderer) {
@@ -1223,7 +1308,7 @@ bool OutputEditorHOCR::save(const std::string& filename) {
 	                           "  </head>\n"
 	                           "<body>\n", tess.Version());
 	file.write(header.data(), header.bytes());
-	for(Gtk::TreeIter item : m_itemStore->children()) {
+	for(Gtk::TreeIter item : m_itemStore->get_iter(m_rootItem)->children()) {
 		Glib::ustring itemSource = (*item)[m_itemStoreCols.source];
 		file.write(itemSource.data(), itemSource.bytes());
 	}
@@ -1307,6 +1392,7 @@ void OutputEditorHOCR::savePDF() {
 
 	PDFSettings pdfSettings;
 	pdfSettings.colorFormat = (*m_builder("combo:pdfoptions.imageformat").as<Gtk::ComboBox>()->get_active())[m_formatComboCols.format];
+	pdfSettings.conversionFlags = pdfSettings.colorFormat == Image::Format_Mono ? (*m_builder("combo:pdfoptions.dithering").as<Gtk::ComboBox>()->get_active())[m_ditheringComboCols.conversionFlags] : Image::AutoColor;
 	pdfSettings.compression = (*m_builder("combo:pdfoptions.compression").as<Gtk::ComboBox>()->get_active())[m_compressionComboCols.mode];
 	pdfSettings.compressionQuality = m_builder("spin:pdfoptions.quality").as<Gtk::SpinButton>()->get_value();
 	pdfSettings.useDetectedFontSizes = m_builder("checkbox:pdfoptions.usedetectedfontsizes").as<Gtk::CheckButton>()->get_active();
@@ -1315,7 +1401,7 @@ void OutputEditorHOCR::savePDF() {
 	pdfSettings.overlay = m_builder("combo:pdfoptions.mode").as<Gtk::ComboBox>()->get_active_row_number() == 1;
 	pdfSettings.detectedFontScaling = m_builder("spin:pdfoptions.fontscale").as<Gtk::SpinButton>()->get_value() / 100.;
 	std::vector<Glib::ustring> failed;
-	for(Gtk::TreeIter item : m_itemStore->children()) {
+	for(Gtk::TreeIter item : m_itemStore->get_iter(m_rootItem)->children()) {
 		if(!(*item)[m_itemStoreCols.selected]) {
 			continue;
 		}
@@ -1323,20 +1409,23 @@ void OutputEditorHOCR::savePDF() {
 		xmlpp::DomParser parser;
 		parser.parse_memory((*item)[m_itemStoreCols.source]);
 		xmlpp::Document* doc = parser.get_document();
-		int pageDpi = 72;
-		if(doc->get_root_node() && doc->get_root_node()->get_name() == "div" && setCurrentSource(doc->get_root_node(), &pageDpi)) {
-			double dpiScale = 72. / pageDpi;
-			double imageScale = m_builder("spin:pdfoptions.dpi").as<Gtk::SpinButton>()->get_value() / double(pageDpi);
-			PoDoFo::PdfPage* page = document->CreatePage(PoDoFo::PdfRect(0, 0, bbox.width * dpiScale, bbox.height * dpiScale));
+		int sourceDpi = -1;
+		int outputDpi = m_builder("spin:pdfoptions.dpi").as<Gtk::SpinButton>()->get_value();
+		if(doc->get_root_node() && doc->get_root_node()->get_name() == "div" && setCurrentSource(doc->get_root_node(), &sourceDpi, &outputDpi)) {
+			double docScale = (72. / sourceDpi);
+			double imgScale = double(outputDpi) / sourceDpi;
+			PoDoFo::PdfPage* page = document->CreatePage(PoDoFo::PdfRect(0, 0, bbox.width * docScale, bbox.height * docScale));
 			painter.SetPage(page);
 			painter.SetFont(font);
 
-			PoDoFoPDFPainter pdfprinter(document, &painter, dpiScale, imageScale);
+			PoDoFoPDFPainter pdfprinter(document, &painter, docScale);
 			pdfprinter.setFontSize(fontSize);
-			printChildren(pdfprinter, item, pdfSettings);
+			printChildren(pdfprinter, item, pdfSettings, imgScale);
 			if(pdfSettings.overlay) {
-				pdfprinter.drawImage(bbox, m_tool->getSelection(bbox), pdfSettings);
+				Geometry::Rectangle scaledBBox(imgScale * bbox.x, imgScale * bbox.y, imgScale * bbox.width, imgScale * bbox.height);
+				pdfprinter.drawImage(bbox, m_tool->getSelection(scaledBBox), pdfSettings);
 			}
+			MAIN->getDisplayer()->setResolution(sourceDpi);
 			painter.FinishPage();
 		} else {
 			failed.push_back((*item)[m_itemStoreCols.text]);
@@ -1349,7 +1438,7 @@ void OutputEditorHOCR::savePDF() {
 	delete document;
 }
 
-void OutputEditorHOCR::printChildren(PDFPainter& painter, Gtk::TreeIter item, const PDFSettings& pdfSettings) const {
+void OutputEditorHOCR::printChildren(PDFPainter& painter, Gtk::TreeIter item, const PDFSettings& pdfSettings, double imgScale) const {
 	if(!(*item)[m_itemStoreCols.selected]) {
 		return;
 	}
@@ -1390,11 +1479,11 @@ void OutputEditorHOCR::printChildren(PDFPainter& painter, Gtk::TreeIter item, co
 			painter.drawText(wordRect.x, y, Glib::ustring((*wordItem)[m_itemStoreCols.text]));
 		}
 	} else if(itemClass == "ocr_graphic" && !pdfSettings.overlay) {
-		Cairo::RefPtr<Cairo::ImageSurface> sel = m_tool->getSelection(itemRect);
-		painter.drawImage(itemRect, sel, pdfSettings);
+		Geometry::Rectangle scaledItemRect(imgScale * itemRect.x, imgScale * itemRect.y, imgScale * itemRect.width, imgScale * itemRect.height);
+		painter.drawImage(itemRect, m_tool->getSelection(scaledItemRect), pdfSettings);
 	} else {
 		for(Gtk::TreeIter child : item->children()) {
-			printChildren(painter, child, pdfSettings);
+			printChildren(painter, child, pdfSettings, imgScale);
 		}
 	}
 }
@@ -1405,12 +1494,12 @@ void OutputEditorHOCR::updatePreview() {
 	}
 	bool visible = m_builder("checkbox:pdfoptions.preview").as<Gtk::CheckButton>()->get_active();
 	m_preview->setVisible(visible);
-	if(m_itemStore->children().empty()|| !visible) {
+	if(m_itemStore->get_iter(m_rootItem)->children().empty()|| !visible) {
 		return;
 	}
 	Gtk::TreeIter item = currentItem();
 	if(!item) {
-		item = *m_itemStore->children().begin();
+		item = *m_itemStore->get_iter(m_rootItem)->children().begin();
 	} else {
 		while(item->parent()) {
 			item = item->parent();
@@ -1421,7 +1510,7 @@ void OutputEditorHOCR::updatePreview() {
 	xmlpp::DomParser parser;
 	parser.parse_memory((*item)[m_itemStoreCols.source]);
 	xmlpp::Document* doc = parser.get_document();
-	int pageDpi = 72;
+	int pageDpi = -1;
 	setCurrentSource(doc->get_root_node(), &pageDpi);
 
 	Cairo::RefPtr<Cairo::ImageSurface> image = Cairo::ImageSurface::create(Cairo::FORMAT_ARGB32, bbox.width, bbox.height);
@@ -1437,6 +1526,7 @@ void OutputEditorHOCR::updatePreview() {
 
 	PDFSettings pdfSettings;
 	pdfSettings.colorFormat = (*m_builder("combo:pdfoptions.imageformat").as<Gtk::ComboBox>()->get_active())[m_formatComboCols.format];
+	pdfSettings.conversionFlags = pdfSettings.colorFormat == Image::Format_Mono ? (*m_builder("combo:pdfoptions.dithering").as<Gtk::ComboBox>()->get_active())[m_ditheringComboCols.conversionFlags] : Image::AutoColor;
 	pdfSettings.compression = (*m_builder("combo:pdfoptions.compression").as<Gtk::ComboBox>()->get_active())[m_compressionComboCols.mode];
 	pdfSettings.compressionQuality = m_builder("spin:pdfoptions.quality").as<Gtk::SpinButton>()->get_value();
 	pdfSettings.useDetectedFontSizes = m_builder("checkbox:pdfoptions.usedetectedfontsizes").as<Gtk::CheckButton>()->get_active();
@@ -1480,12 +1570,17 @@ bool OutputEditorHOCR::clear(bool hide) {
 	}
 	m_idCounter = 0;
 	m_connectionSelectionChanged.block();
-	m_itemStore->clear();
+	Gtk::TreeIter rootIter = m_itemStore->get_iter(m_rootItem);
+	while(!rootIter->children().empty()) {
+		m_itemStore->erase(*rootIter->children()[0]);
+	}
 	m_connectionSelectionChanged.unblock();
 	m_propStore->clear();
 	m_sourceView->get_buffer()->set_text("");
 	m_tool->clearSelection();
 	m_modified = false;
+	m_builder("button:hocr.save")->set_sensitive(false);
+	m_builder("button:hocr.export")->set_sensitive(false);
 	if(hide)
 		MAIN->setOutputPaneVisible(false);
 	return true;
