@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * OutputEditorText.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -59,6 +59,12 @@ OutputEditorText::OutputEditorText()
 	m_toggleSearchButton->add_accelerator("clicked", group, GDK_KEY_F, Gdk::CONTROL_MASK, Gtk::AccelFlags(0));
 	saveButton->add_accelerator("clicked", group, GDK_KEY_S, Gdk::CONTROL_MASK, Gtk::AccelFlags(0));
 
+	m_builder("image:output.insert").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/ins_append.png"));
+	m_builder("image:output.stripcrlf").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/stripcrlf.png"));
+	m_builder("image:output.insert.append").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/ins_append.png"));
+	m_builder("image:output.insert.cursor").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/ins_cursor.png"));
+	m_builder("image:output.insert.replace").as<Gtk::Image>()->set(Gdk::Pixbuf::create_from_resource("/org/gnome/gimagereader/ins_replace.png"));
+
 	m_substitutionsManager = new SubstitutionsManager(m_builder, m_textBuffer);
 
 	m_insertMode = InsertMode::Append;
@@ -169,8 +175,8 @@ void OutputEditorText::filterBuffer() {
 			preChars += "\\.\\?!"; // Keep if preceded by end mark (.?!)
 		}
 		if(m_filterKeepIfQuote->get_active()) {
-			preChars += "'\""; // Keep if preceded by dot
-			sucChars += "'\""; // Keep if succeeded by dot
+			preChars += "'\"\u00BB\u00AB"; // Keep if preceded by quote
+			sucChars += "'\"\u00AB\u00BB"; // Keep if succeeded by quote
 		}
 		if(m_filterKeepParagraphs->get_active()) {
 			sucChars += "\\n"; // Keep if succeeded by line break
@@ -248,10 +254,24 @@ void OutputEditorText::findReplace(bool backwards, bool replace) {
 }
 
 void OutputEditorText::read(tesseract::TessBaseAPI &tess, ReadSessionData *data) {
-	char* text = tess.GetUTF8Text();
+	char* textbuf = tess.GetUTF8Text();
+	Glib::ustring text = Glib::ustring(textbuf);
+	if(!text.empty() && *--text.end() != '\n')
+		text.append("\n");
+	if(data->prependFile || data->prependPage) {
+		std::vector<Glib::ustring> prepend;
+		if(data->prependFile) {
+			prepend.push_back(Glib::ustring::compose(_("File: %1"), data->file));
+		}
+		if(data->prependPage) {
+			prepend.push_back(Glib::ustring::compose(_("Page: %1"), data->page));
+		}
+		text = Glib::ustring::compose("[%1]\n", Utils::string_join(prepend, "; ")) + text;
+	}
+
 	bool& insertText = static_cast<TextReadSessionData*>(data)->insertText;
 	Utils::runInMainThreadBlocking([&] { addText(text, insertText); });
-	delete[] text;
+	delete[] textbuf;
 	insertText = true;
 }
 
