@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Recognizer.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -121,7 +121,9 @@ Recognizer::Recognizer() {
 
 	MAIN->getConfig()->addSetting(new VarSetting<Glib::ustring>("language"));
 	MAIN->getConfig()->addSetting(new ComboSetting("ocrregionstrategy", MAIN->getWidget("comboboxtext:dialog.regions")));
-	MAIN->getConfig()->addSetting(new VarSetting<bool>("osd"));
+	MAIN->getConfig()->addSetting(new SwitchSettingT<Gtk::CheckButton>("ocraddsourcefilename", MAIN->getWidget("checkbutton:dialog.pages.prepend.filename")));
+	MAIN->getConfig()->addSetting(new SwitchSettingT<Gtk::CheckButton>("ocraddsourcepage", MAIN->getWidget("checkbutton:dialog.pages.prepend.page")));
+	MAIN->getConfig()->addSetting(new VarSetting<int>("psm"));
 }
 
 std::vector<Glib::ustring> Recognizer::getAvailableLanguages() const {
@@ -193,7 +195,7 @@ void Recognizer::updateLanguagesMenu() {
 	});
 	m_langMenuRadioGroup = Gtk::RadioButtonGroup();
 	m_langMenuCheckGroup = std::vector<std::pair<Gtk::CheckMenuItem*, Glib::ustring>>();
-	m_osdItem = nullptr;
+	m_psmRadioGroup = Gtk::RadioButtonGroup();
 	m_curLang = Config::Lang();
 	Gtk::RadioMenuItem* curitem = nullptr;
 	Gtk::RadioMenuItem* activeitem = nullptr;
@@ -302,15 +304,44 @@ void Recognizer::updateLanguagesMenu() {
 		activeitem = curitem;
 	}
 
-	// Add OSD item
-	if(haveOsd) {
-		m_menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
-		m_osdItem = Gtk::manage(new Gtk::CheckMenuItem(_("Detect script and orientation")));
-		m_osdItem->set_active(MAIN->getConfig()->getSetting<VarSetting<bool>>("osd")->getValue());
-		CONNECT(m_osdItem, toggled, [this] { MAIN->getConfig()->getSetting<VarSetting<bool>>("osd")->setValue(m_osdItem->get_active()); });
-		m_menuLanguages->append(*m_osdItem);
+	// Add PSM items
+	m_menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
+	Gtk::Menu* psmMenu = Gtk::manage(new Gtk::Menu);
+	m_currentPsmMode = MAIN->getConfig()->getSetting<VarSetting<int>>("psm")->getValue();
+
+	struct PsmEntry {
+		Glib::ustring label;
+		tesseract::PageSegMode psmMode;
+		bool requireOsd;
+	};
+	std::vector<PsmEntry> psmModes = {
+		PsmEntry{_("Automatic page segmentation"), tesseract::PSM_AUTO, false},
+		PsmEntry{_("Page segmentation with orientation and script detection"), tesseract::PSM_AUTO_OSD, true},
+		PsmEntry{_("Assume single column of text"), tesseract::PSM_SINGLE_COLUMN, false},
+		PsmEntry{_("Assume single block of vertically aligned text"), tesseract::PSM_SINGLE_BLOCK_VERT_TEXT, false},
+		PsmEntry{_("Assume a single uniform block of text"), tesseract::PSM_SINGLE_BLOCK, false},
+		PsmEntry{_("Assume a line of text"), tesseract::PSM_SINGLE_LINE, false},
+		PsmEntry{_("Assume a single word"), tesseract::PSM_SINGLE_WORD, false},
+		PsmEntry{_("Assume a single word in a circle"), tesseract::PSM_CIRCLE_WORD, false},
+		PsmEntry{_("Sparse text in no particular order"), tesseract::PSM_SPARSE_TEXT, false},
+		PsmEntry{_("Sparse text with orientation and script detection"), tesseract::PSM_SPARSE_TEXT_OSD, true}};
+	for(const auto& entry : psmModes) {
+		Gtk::RadioMenuItem* item = Gtk::manage(new Gtk::RadioMenuItem(m_psmRadioGroup, entry.label));
+		item->set_sensitive(!entry.requireOsd || haveOsd);
+		item->set_active(entry.psmMode == m_currentPsmMode);
+		CONNECT(item, toggled, [this, entry, item] {
+			if(item->get_active()) {
+				m_currentPsmMode = entry.psmMode;
+				MAIN->getConfig()->getSetting<VarSetting<int>>("psm")->setValue(entry.psmMode);
+			}
+		});
+		psmMenu->append(*item);
 	}
 
+	Gtk::MenuItem* psmItem = Gtk::manage(new Gtk::MenuItem(_("Page segmentation mode")));
+	psmItem->set_submenu(*psmMenu);
+	m_menuLanguages->append(*psmItem);
+
 	// Add installer item
 	m_menuLanguages->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
 	Gtk::MenuItem* manageItem = Gtk::manage(new Gtk::MenuItem(_("Manage languages...")));
@@ -366,6 +397,7 @@ std::vector<int> Recognizer::selectPages(bool& autodetectLayout) {
 	m_pagesEntry->grab_focus();
 	m_pageAreaLabel->set_visible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
 	m_pageAreaCombo->set_visible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
+	MAIN->getWidget("box:dialog.pages.prepend")->set_visible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
 	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(m_pageAreaCombo->get_model());
 	int col = m_pageAreaCombo->get_entry_text_column();
 	store->children()[0]->set_value<Glib::ustring>(col, MAIN->getDisplayer()->hasMultipleOCRAreas() ? _("Current selection") : _("Entire page"));
@@ -425,17 +457,18 @@ void Recognizer::recognizeMultiplePages() {
 
 void Recognizer::recognize(const std::vector<int> &pages, bool autodetectLayout) {
 	tesseract::TessBaseAPI tess;
+	bool prependFile = pages.size() > 1 && MAIN->getConfig()->getSetting<SwitchSetting>("ocraddsourcefilename")->getValue();
+	bool prependPage = pages.size() > 1 && MAIN->getConfig()->getSetting<SwitchSetting>("ocraddsourcepage")->getValue();
 	if(initTesseract(tess, m_curLang.prefix.c_str())) {
 		Glib::ustring failed;
-		if(MAIN->getConfig()->getSetting<VarSetting<bool>>("osd")->getValue() == true) {
-			tess.SetPageSegMode(tesseract::PSM_AUTO_OSD);
-		}
+		tess.SetPageSegMode(static_cast<tesseract::PageSegMode>(m_currentPsmMode));
 		OutputEditor::ReadSessionData* readSessionData = MAIN->getOutputEditor()->initRead(tess);
 		ProgressMonitor monitor(pages.size());
 		MAIN->showProgress(&monitor);
 		Utils::busyTask([&] {
 			int npages = pages.size();
 			int idx = 0;
+			std::string prevFile;
 			for(int page : pages) {
 				monitor.desc.progress = 0;
 				++idx;
@@ -451,7 +484,14 @@ void Recognizer::recognize(const std::vector<int> &pages, bool autodetectLayout)
 				readSessionData->file = MAIN->getDisplayer()->getCurrentImage(readSessionData->page);
 				readSessionData->angle = MAIN->getDisplayer()->getCurrentAngle();
 				readSessionData->resolution = MAIN->getDisplayer()->getCurrentResolution();
+				bool firstChunk = true;
+				bool newFile = readSessionData->file != prevFile;
+				prevFile = readSessionData->file;
 				for(const Cairo::RefPtr<Cairo::ImageSurface>& image : MAIN->getDisplayer()->getOCRAreas()) {
+					readSessionData->prependPage = prependPage && firstChunk;
+					readSessionData->prependFile = prependFile && (readSessionData->prependPage || newFile);
+					firstChunk = false;
+					newFile = false;
 					tess.SetImage(image->get_data(), image->get_width(), image->get_height(), 4, image->get_stride());
 					tess.SetSourceResolution(MAIN->getDisplayer()->getCurrentResolution());
 					tess.Recognize(&monitor.desc);
@@ -459,6 +499,7 @@ void Recognizer::recognize(const std::vector<int> &pages, bool autodetectLayout)
 						MAIN->getOutputEditor()->read(tess, readSessionData);
 					}
 				}
+
 				Glib::signal_idle().connect_once([] { MAIN->popState(); });
 				++monitor.donePages;
 				if(monitor.canceled) {
