@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * SourceManager.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -206,8 +206,10 @@ void SourceManager::removeSource(bool deleteFile) {
 	m_connectionSelectionChanged.block(true);
 	Glib::RefPtr<Gtk::ListStore> store = Glib::RefPtr<Gtk::ListStore>::cast_static(m_listView->get_model());
 	Gtk::TreeIter it;
-	for(const Gtk::TreeModel::Path& path : m_listView->get_selection()->get_selected_rows()) {
-		it = m_listView->get_model()->get_iter(path);
+	std::vector<Gtk::TreeModel::Path> selected = m_listView->get_selection()->get_selected_rows();
+	while(!selected.empty()) {
+		it = m_listView->get_model()->get_iter(selected.back());
+		selected.pop_back();
 		Source* source = it->get_value(m_listViewCols.source);
 		if(deleteFile || source->isTemp) {
 			source->file->remove();
@@ -222,7 +224,7 @@ void SourceManager::removeSource(bool deleteFile) {
 		m_listView->get_selection()->select(it);
 	}
 	m_connectionSelectionChanged.block(false);
-	m_signal_sourceChanged.emit();
+	selectionChanged();
 }
 
 void SourceManager::clearSources() {
@@ -237,7 +239,7 @@ void SourceManager::clearSources() {
 	m_connectionSelectionChanged.block(true);
 	store->clear();
 	m_connectionSelectionChanged.block(false);
-	m_signal_sourceChanged.emit();
+	selectionChanged();
 }
 
 void SourceManager::selectionChanged() {
