@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Config.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -334,7 +334,7 @@ Config::Config(QWidget* parent)
 }
 
 Config::~Config() {
-	qDeleteAll(m_settings.values());
+	qDeleteAll(m_settings);
 }
 
 bool Config::searchLangSpec(Lang& lang) const {
