@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * DisplayRenderer.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -67,8 +67,13 @@ void DisplayRenderer::adjustImage(QImage &image, int brightness, int contrast, b
 	}
 }
 
-QImage ImageRenderer::render(int /*page*/, double resolution) const {
+ImageRenderer::ImageRenderer(const QString &filename) : DisplayRenderer(filename) {
+	m_pageCount = QImageReader(m_filename).imageCount();
+}
+
+QImage ImageRenderer::render(int page, double resolution) const {
 	QImageReader reader(m_filename);
+	reader.jumpToImage(page - 1);
 	reader.setBackgroundColor(Qt::white);
 	reader.setScaledSize(reader.size() * resolution / 100.);
 	return reader.read().convertToFormat(QImage::Format_RGB32);
