@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Displayer.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -56,20 +56,25 @@ Displayer::Displayer(const UI_MainWindow& _ui, QWidget* parent)
 	setBackgroundBrush(Qt::gray);
 	setRenderHint(QPainter::Antialiasing);
 
+	m_rotateMode = RotateMode::AllPages;
+	ui.actionRotateCurrentPage->setData(static_cast<int>(RotateMode::CurrentPage));
+	ui.actionRotateAllPages->setData(static_cast<int>(RotateMode::AllPages));
+
 	m_renderTimer.setSingleShot(true);
 	m_scaleTimer.setSingleShot(true);
 
 	ui.actionRotateLeft->setData(270.);
 	ui.actionRotateRight->setData(90.);
 
+	connect(ui.menuRotation, SIGNAL(triggered(QAction*)), this, SLOT(setRotateMode(QAction*)));
 	connect(ui.actionRotateLeft, SIGNAL(triggered()), this, SLOT(rotate90()));
 	connect(ui.actionRotateRight, SIGNAL(triggered()), this, SLOT(rotate90()));
 	connect(ui.spinBoxRotation, SIGNAL(valueChanged(double)), this, SLOT(setAngle(double)));
 	connect(ui.spinBoxPage, SIGNAL(valueChanged(int)), this, SLOT(setCurrentPage(int)));
-	connect(ui.spinBoxBrightness, SIGNAL(valueChanged(int)), this, SLOT(queueRenderImage()));
-	connect(ui.spinBoxContrast, SIGNAL(valueChanged(int)), this, SLOT(queueRenderImage()));
-	connect(ui.spinBoxResolution, SIGNAL(valueChanged(int)), this, SLOT(queueRenderImage()));
-	connect(ui.checkBoxInvertColors, SIGNAL(toggled(bool)), this, SLOT(queueRenderImage()));
+	connect(ui.spinBoxBrightness, SIGNAL(valueChanged(int)), this, SLOT(brightnessChanged()));
+	connect(ui.spinBoxContrast, SIGNAL(valueChanged(int)), this, SLOT(contrastChanged()));
+	connect(ui.spinBoxResolution, SIGNAL(valueChanged(int)), this, SLOT(resolutionChanged()));
+	connect(ui.checkBoxInvertColors, SIGNAL(toggled(bool)), this, SLOT(invertColorsChanged()));
 	connect(ui.actionZoomIn, SIGNAL(triggered()), this, SLOT(zoomIn()));
 	connect(ui.actionZoomOut, SIGNAL(triggered()), this, SLOT(zoomOut()));
 	connect(ui.actionBestFit, SIGNAL(triggered()), this, SLOT(zoomFit()));
@@ -93,6 +98,11 @@ bool Displayer::setCurrentPage(int page) {
 	}
 	Source* source = m_pageMap[page].first;
 	if(source != m_currentSource) {
+		sendScaleRequest({ScaleRequest::Abort});
+		sendScaleRequest({ScaleRequest::Quit});
+		while(m_scaleThread.isRunning()) {
+			QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+		}
 		delete m_renderer;
 		if(source->path.endsWith(".pdf", Qt::CaseInsensitive)) {
 			m_renderer = new PDFRenderer(source->path);
@@ -102,7 +112,6 @@ bool Displayer::setCurrentPage(int page) {
 			if(source->resolution == -1) source->resolution = 100;
 		}
 
-		Utils::setSpinBlocked(ui.spinBoxRotation, source->angle);
 		Utils::setSpinBlocked(ui.spinBoxBrightness, source->brightness);
 		Utils::setSpinBlocked(ui.spinBoxContrast, source->contrast);
 		Utils::setSpinBlocked(ui.spinBoxResolution, source->resolution);
@@ -110,10 +119,10 @@ bool Displayer::setCurrentPage(int page) {
 		ui.checkBoxInvertColors->setChecked(source->invert);
 		ui.checkBoxInvertColors->blockSignals(false);
 		m_currentSource = source;
+		m_scaleThread.start();
 	}
-	ui.spinBoxPage->blockSignals(true);
-	ui.spinBoxPage->setValue(page);
-	ui.spinBoxPage->blockSignals(false);
+	Utils::setSpinBlocked(ui.spinBoxRotation, source->angle[m_pageMap[page].second - 1]);
+	Utils::setSpinBlocked(ui.spinBoxPage, page);
 
 	bool result = renderImage();
 	ui.spinBoxPage->setEnabled(true);
@@ -148,8 +157,11 @@ bool Displayer::setSources(QList<Source*> sources) {
 
 	m_scaleTimer.stop();
 	if(m_scaleThread.isRunning()) {
+		sendScaleRequest({ScaleRequest::Abort});
 		sendScaleRequest({ScaleRequest::Quit});
-		m_scaleThread.wait();
+		while(m_scaleThread.isRunning()) {
+			QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+		}
 	}
 	if(m_tool) {
 		m_tool->reset();
@@ -187,17 +199,27 @@ bool Displayer::setSources(QList<Source*> sources) {
 
 	int page = 0;
 	for(Source* source : m_sources) {
+		DisplayRenderer* renderer;
 		if(source->path.endsWith(".pdf", Qt::CaseInsensitive)) {
-			PDFRenderer r(source->path);
-			for(int pdfPage = 1, nPdfPages = r.getNPages(); pdfPage <= nPdfPages; ++pdfPage) {
-				m_pageMap.insert(++page, qMakePair(source, pdfPage));
-			}
+			renderer = new PDFRenderer(source->path);
 		} else {
-			m_pageMap.insert(++page, qMakePair(source, 1));
+			renderer = new ImageRenderer(source->path);
 		}
+		source->angle.resize(renderer->getNPages());
+		for(int iPage = 1, nPages = renderer->getNPages(); iPage <= nPages; ++iPage) {
+			m_pageMap.insert(++page, qMakePair(source, iPage));
+		}
+		delete renderer;
+	}
+	if(page == 0) {
+		m_pageMap.clear();
+		m_sources.clear();
+		return false;
 	}
 
+	ui.spinBoxPage->blockSignals(true);
 	ui.spinBoxPage->setMaximum(page);
+	ui.spinBoxPage->blockSignals(false);
 	ui.actionPage->setVisible(page > 1);
 	setCursor(Qt::CrossCursor);
 	m_imageItem = new QGraphicsPixmapItem();
@@ -311,8 +333,16 @@ void Displayer::setAngle(double angle) {
 	if(m_imageItem) {
 		angle = angle < 0 ? angle + 360. : angle >= 360 ? angle - 360 : angle,
 		Utils::setSpinBlocked(ui.spinBoxRotation, angle);
-		double delta = angle - m_currentSource->angle;
-		m_currentSource->angle = angle;
+		int sourcePage = m_pageMap[getCurrentPage()].second;
+		double delta = angle - m_currentSource->angle[sourcePage - 1];
+		if(m_rotateMode == RotateMode::CurrentPage) {
+			m_currentSource->angle[sourcePage - 1] = angle;
+		} else if(delta != 0) {
+			for(int page : m_pageMap.keys()) {
+				auto pair = m_pageMap[page];
+				pair.first->angle[pair.second - 1] += delta;
+			}
+		}
 		m_imageItem->setRotation(angle);
 		if(m_tool) {
 			m_tool->rotationChanged(delta);
@@ -324,10 +354,41 @@ void Displayer::setAngle(double angle) {
 	}
 }
 
+
+void Displayer::brightnessChanged() {
+	int brightness = ui.spinBoxBrightness->value();
+	for(int page : m_pageMap.keys()) {
+		m_pageMap[page].first->brightness = brightness;
+	}
+	queueRenderImage();
+}
+
+void Displayer::contrastChanged() {
+	int contrast = ui.spinBoxBrightness->value();
+	for(int page : m_pageMap.keys()) {
+		m_pageMap[page].first->contrast = contrast;
+	}
+	queueRenderImage();
+}
+
+void Displayer::resolutionChanged() {
+	int resolution = ui.spinBoxResolution->value();
+	for(int page : m_pageMap.keys()) {
+		m_pageMap[page].first->resolution = resolution;
+	}
+	queueRenderImage();
+}
+
+void Displayer::invertColorsChanged() {
+	bool invert = ui.checkBoxInvertColors->isChecked();
+	for(int page : m_pageMap.keys()) {
+		m_pageMap[page].first->invert = invert;
+	}
+	queueRenderImage();
+}
+
 void Displayer::setResolution(int resolution) {
-	ui.spinBoxResolution->blockSignals(true);
-	ui.spinBoxResolution->setValue(resolution);
-	ui.spinBoxResolution->blockSignals(false);
+	Utils::setSpinBlocked(ui.spinBoxResolution, resolution);
 	renderImage();
 }
 
@@ -417,6 +478,11 @@ QPointF Displayer::mapToSceneClamped(const QPoint &p) const {
 	return q;
 }
 
+void Displayer::setRotateMode(QAction *action) {
+	m_rotateMode = static_cast<RotateMode>(action->data().value<int>());
+	ui.toolButtonRotation->setIcon(action->icon());
+}
+
 void Displayer::rotate90() {
 	double angle = ui.spinBoxRotation->value() + qobject_cast<QAction*>(QObject::sender())->data().toDouble();
 	ui.spinBoxRotation->setValue(angle >= 360. ? angle - 360. : angle);
@@ -468,6 +534,9 @@ void Displayer::scaleThread() {
 		} else if(req.type == ScaleRequest::Scale) {
 			m_scaleMutex.unlock();
 			QImage image = m_renderer->render(req.page, req.scale * req.resolution);
+			if(image.isNull()) {
+				continue;
+			}
 
 			m_scaleMutex.lock();
 			if(!m_scaleRequests.isEmpty() && m_scaleRequests.first().type == ScaleRequest::Abort) {
