@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * MainWindow.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -34,6 +34,7 @@
 #include <sys/prctl.h>
 #endif
 #include <QtSpell.hpp>
+#include <tesseract/baseapi.h>
 
 #include "MainWindow.hh"
 #include "Acquirer.hh"
@@ -270,6 +271,7 @@ void MainWindow::showAbout() {
 	Ui::AboutDialog aboutDialogUi;
 	aboutDialogUi.setupUi(&d);
 	aboutDialogUi.labelVersion->setText(PACKAGE_VERSION);
+	aboutDialogUi.labelTesseractVer->setText(QString("<html><head/><body><p style=\"font-size:small;\">%1 %2</p></body></html>").arg(_("Using tesseract")).arg(TESSERACT_VERSION_STR));
 	d.exec();
 }
 
