@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * OutputEditorHOCR.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -27,6 +27,7 @@
 #include <QFileInfo>
 #include <QFileDialog>
 #include <QPainter>
+#include <QStandardItemModel>
 #include <QSyntaxHighlighter>
 #include <QTextStream>
 #include <cstring>
@@ -42,6 +43,7 @@
 #include <tesseract/baseapi.h>
 #include <tesseract/ocrclass.h>
 
+#include "CCITTFax4Encoder.hh"
 #include "DisplayerToolHOCR.hh"
 #include "MainWindow.hh"
 #include "OutputEditorHOCR.hh"
@@ -134,7 +136,14 @@ public:
 		m_painter->drawText(x, y, text);
 	}
 	void drawImage(const QRect& bbox, const QImage& image, const PDFSettings& settings) override {
-		m_painter->drawImage(bbox, convertedImage(image, settings.colorFormat));
+		QImage img = convertedImage(image, settings.colorFormat, settings.conversionFlags);
+		if(settings.compression == PDFSettings::CompressJpeg) {
+			QByteArray data;
+			QBuffer buffer(&data);
+			img.save(&buffer, "jpg", settings.compressionQuality);
+			img = QImage::fromData(data);
+		}
+		m_painter->drawImage(bbox, img);
 	}
 	double getAverageCharWidth() const override {
 		return m_painter->fontMetrics().averageCharWidth();
@@ -174,8 +183,8 @@ public:
 
 class OutputEditorHOCR::PoDoFoPDFPainter : public OutputEditorHOCR::PDFPainter {
 public:
-	PoDoFoPDFPainter(PoDoFo::PdfDocument* document, PoDoFo::PdfPainter* painter, double scaleFactor, double imageScale)
-		: m_document(document), m_painter(painter), m_scaleFactor(scaleFactor), m_imageScale(imageScale) {
+	PoDoFoPDFPainter(PoDoFo::PdfDocument* document, PoDoFo::PdfPainter* painter, double scaleFactor)
+		: m_document(document), m_painter(painter), m_scaleFactor(scaleFactor) {
 		m_pageHeight = m_painter->GetPage()->GetPageSize().GetHeight();
 	}
 	void setFontSize(double pointSize) override {
@@ -186,8 +195,7 @@ public:
 		m_painter->DrawText(x * m_scaleFactor, m_pageHeight - y * m_scaleFactor, pdfString);
 	}
 	void drawImage(const QRect& bbox, const QImage& image, const PDFSettings& settings) override {
-		QImage scaledImage = image.scaled(image.width() * m_imageScale, image.height() * m_imageScale, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-		QImage img = convertedImage(scaledImage, settings.colorFormat);
+		QImage img = convertedImage(image, settings.colorFormat, settings.conversionFlags);
 		if(settings.colorFormat == QImage::Format_Mono) {
 			img.invertPixels();
 		}
@@ -210,7 +218,7 @@ public:
 			}
 			PoDoFo::PdfMemoryInputStream is(buf.data(), bytesPerLine * height);
 			pdfImage.SetImageData(width, height, sampleSize, &is, {PoDoFo::ePdfFilter_FlateDecode});
-		} else {
+		} else if(settings.compression == PDFSettings::CompressJpeg) {
 			PoDoFo::PdfName dctFilterName(PoDoFo::PdfFilterFactory::FilterTypeToName(PoDoFo::ePdfFilter_DCTDecode));
 			pdfImage.GetObject()->GetDictionary().AddKey(PoDoFo::PdfName::KeyFilter, dctFilterName);
 			QByteArray data;
@@ -218,8 +226,21 @@ public:
 			img.save(&buffer, "jpg", settings.compressionQuality);
 			PoDoFo::PdfMemoryInputStream is(data.data(), data.size());
 			pdfImage.SetImageDataRaw(width, height, sampleSize, &is);
+		} else if(settings.compression == PDFSettings::CompressFax4) {
+			PoDoFo::PdfName faxFilterName(PoDoFo::PdfFilterFactory::FilterTypeToName(PoDoFo::ePdfFilter_CCITTFaxDecode));
+			pdfImage.GetObject()->GetDictionary().AddKey(PoDoFo::PdfName::KeyFilter, faxFilterName);
+			PoDoFo::PdfDictionary decodeParams;
+			decodeParams.AddKey("Columns", PoDoFo::PdfObject(PoDoFo::pdf_int64(img.width())));
+			decodeParams.AddKey("Rows", PoDoFo::PdfObject(PoDoFo::pdf_int64(img.height())));
+			decodeParams.AddKey("K", PoDoFo::PdfObject(PoDoFo::pdf_int64(-1))); // K < 0 --- Pure two-dimensional encoding (Group 4)
+			pdfImage.GetObject()->GetDictionary().AddKey("DecodeParms", PoDoFo::PdfObject(decodeParams));
+			CCITTFax4Encoder encoder;
+			uint32_t encodedLen = 0;
+			uint8_t* encoded = encoder.encode(img.constBits(), img.width(), img.height(), img.bytesPerLine(), encodedLen);
+			PoDoFo::PdfMemoryInputStream is(reinterpret_cast<char*>(encoded), encodedLen);
+			pdfImage.SetImageDataRaw(img.width(), img.height(), sampleSize, &is);
 		}
-		m_painter->DrawImage(bbox.x() * m_scaleFactor, m_pageHeight - (bbox.y() + bbox.height()) * m_scaleFactor, &pdfImage, m_scaleFactor / m_imageScale, m_scaleFactor / m_imageScale);
+		m_painter->DrawImage(bbox.x() * m_scaleFactor, m_pageHeight - (bbox.y() + bbox.height()) * m_scaleFactor, &pdfImage, m_scaleFactor * bbox.width() / double(image.width()), m_scaleFactor * bbox.height() / double(image.height()));
 	}
 	double getAverageCharWidth() const override {
 		return m_painter->GetFont()->GetFontMetrics()->CharWidth(static_cast<unsigned char>('x')) / m_scaleFactor;
@@ -233,7 +254,6 @@ private:
 	PoDoFo::PdfDocument* m_document;
 	PoDoFo::PdfPainter* m_painter;
 	double m_scaleFactor;
-	double m_imageScale;
 	double m_pageHeight;
 };
 
@@ -258,13 +278,19 @@ OutputEditorHOCR::OutputEditorHOCR(DisplayerToolHOCR* tool) {
 #endif
 	m_pdfExportDialogUi.comboBoxImageFormat->addItem(_("Monochrome"), QImage::Format_Mono);
 	m_pdfExportDialogUi.comboBoxImageFormat->setCurrentIndex(-1);
+	m_pdfExportDialogUi.comboBoxDithering->addItem(_("Threshold (closest color)"), Qt::ThresholdDither);
+	m_pdfExportDialogUi.comboBoxDithering->addItem(_("Diffuse"), Qt::DiffuseDither);
 	m_pdfExportDialogUi.comboBoxImageCompression->addItem(_("Zip (lossless)"), PDFSettings::CompressZip);
+	m_pdfExportDialogUi.comboBoxImageCompression->addItem(_("CCITT Group 4 (lossless)"), PDFSettings::CompressFax4);
 	m_pdfExportDialogUi.comboBoxImageCompression->addItem(_("Jpeg (lossy)"), PDFSettings::CompressJpeg);
 	m_pdfExportDialogUi.comboBoxImageCompression->setCurrentIndex(-1);
 
 	ui.actionOutputSaveHOCR->setShortcut(Qt::CTRL + Qt::Key_S);
 
 	ui.treeWidgetItems->setContextMenuPolicy(Qt::CustomContextMenu);
+	m_rootItem = new QTreeWidgetItem(QStringList() << _("Document"));
+	ui.treeWidgetItems->addTopLevelItem(m_rootItem);
+	m_rootItem->setExpanded(true);
 
 	connect(ui.actionOutputOpen, SIGNAL(triggered()), this, SLOT(open()));
 	connect(ui.actionOutputSaveHOCR, SIGNAL(triggered()), this, SLOT(save()));
@@ -281,7 +307,9 @@ OutputEditorHOCR::OutputEditorHOCR(DisplayerToolHOCR* tool) {
 	connect(m_pdfExportDialogUi.comboBoxOutputMode, SIGNAL(currentIndexChanged(int)), this, SLOT(updatePreview()));
 	connect(m_pdfExportDialogUi.comboBoxImageFormat, SIGNAL(currentIndexChanged(int)), this, SLOT(updatePreview()));
 	connect(m_pdfExportDialogUi.comboBoxImageFormat, SIGNAL(currentIndexChanged(int)), this, SLOT(imageFormatChanged()));
+	connect(m_pdfExportDialogUi.comboBoxDithering, SIGNAL(currentIndexChanged(int)), this, SLOT(updatePreview()));
 	connect(m_pdfExportDialogUi.comboBoxImageCompression, SIGNAL(currentIndexChanged(int)), this, SLOT(imageCompressionChanged()));
+	connect(m_pdfExportDialogUi.spinBoxCompressionQuality, SIGNAL(valueChanged(int)), this, SLOT(updatePreview()));
 	connect(m_pdfExportDialogUi.checkBoxFontSize, SIGNAL(toggled(bool)), this, SLOT(updatePreview()));
 	connect(m_pdfExportDialogUi.checkBoxFontSize, SIGNAL(toggled(bool)), m_pdfExportDialogUi.labelFontScaling, SLOT(setEnabled(bool)));
 	connect(m_pdfExportDialogUi.checkBoxFontSize, SIGNAL(toggled(bool)), m_pdfExportDialogUi.spinFontScaling, SLOT(setEnabled(bool)));
@@ -300,6 +328,7 @@ OutputEditorHOCR::OutputEditorHOCR(DisplayerToolHOCR* tool) {
 	MAIN->getConfig()->addSetting(new SpinSetting("pdfimagecompressionquality", m_pdfExportDialogUi.spinBoxCompressionQuality, 90));
 	MAIN->getConfig()->addSetting(new ComboSetting("pdfimagecompression", m_pdfExportDialogUi.comboBoxImageCompression));
 	MAIN->getConfig()->addSetting(new ComboSetting("pdfimageformat", m_pdfExportDialogUi.comboBoxImageFormat));
+	MAIN->getConfig()->addSetting(new ComboSetting("pdfimageconversionflags", m_pdfExportDialogUi.comboBoxDithering));
 	MAIN->getConfig()->addSetting(new SpinSetting("pdfimagedpi", m_pdfExportDialogUi.spinBoxDpi, 300));
 	MAIN->getConfig()->addSetting(new SwitchSetting("pdfusedetectedfontsizes", m_pdfExportDialogUi.checkBoxFontSize, true));
 	MAIN->getConfig()->addSetting(new SpinSetting("pdffontscale", m_pdfExportDialogUi.spinFontScaling, 100));
@@ -346,21 +375,36 @@ void OutputEditorHOCR::setFont() {
 
 void OutputEditorHOCR::imageFormatChanged() {
 	QImage::Format format = static_cast<QImage::Format>(m_pdfExportDialogUi.comboBoxImageFormat->itemData(m_pdfExportDialogUi.comboBoxImageFormat->currentIndex()).toInt());
+	QStandardItemModel* model = static_cast<QStandardItemModel*>(m_pdfExportDialogUi.comboBoxImageCompression->model());
+	int zipIdx = m_pdfExportDialogUi.comboBoxImageCompression->findData(PDFSettings::CompressZip);
+	int ccittIdx = m_pdfExportDialogUi.comboBoxImageCompression->findData(PDFSettings::CompressFax4);
+	int jpegIdx = m_pdfExportDialogUi.comboBoxImageCompression->findData(PDFSettings::CompressJpeg);
+	QStandardItem* ccittItem = model->item(ccittIdx);
+	QStandardItem* jpegItem = model->item(jpegIdx);
 	if(format == QImage::Format_Mono) {
-		m_pdfExportDialogUi.comboBoxImageCompression->setCurrentIndex(m_pdfExportDialogUi.comboBoxImageCompression->findData(PDFSettings::CompressZip));
-		m_pdfExportDialogUi.comboBoxImageCompression->setEnabled(false);
-		m_pdfExportDialogUi.labelImageCompression->setEnabled(false);
+		if(m_pdfExportDialogUi.comboBoxImageCompression->currentIndex() == jpegIdx) {
+			m_pdfExportDialogUi.comboBoxImageCompression->setCurrentIndex(zipIdx);
+		}
+		ccittItem->setFlags(ccittItem->flags()|Qt::ItemIsSelectable|Qt::ItemIsEnabled);
+		jpegItem->setFlags(jpegItem->flags() & ~(Qt::ItemIsSelectable|Qt::ItemIsEnabled));
+		m_pdfExportDialogUi.labelDithering->setEnabled(true);
+		m_pdfExportDialogUi.comboBoxDithering->setEnabled(true);
 	} else {
-		m_pdfExportDialogUi.comboBoxImageCompression->setEnabled(true);
-		m_pdfExportDialogUi.labelImageCompression->setEnabled(true);
+		if(m_pdfExportDialogUi.comboBoxImageCompression->currentIndex() == ccittIdx) {
+			m_pdfExportDialogUi.comboBoxImageCompression->setCurrentIndex(zipIdx);
+		}
+		ccittItem->setFlags(ccittItem->flags() & ~(Qt::ItemIsSelectable|Qt::ItemIsEnabled));
+		jpegItem->setFlags(jpegItem->flags()|Qt::ItemIsSelectable|Qt::ItemIsEnabled);
+		m_pdfExportDialogUi.labelDithering->setEnabled(false);
+		m_pdfExportDialogUi.comboBoxDithering->setEnabled(false);
 	}
 }
 
 void OutputEditorHOCR::imageCompressionChanged() {
 	PDFSettings::Compression compression = static_cast<PDFSettings::Compression>(m_pdfExportDialogUi.comboBoxImageCompression->itemData(m_pdfExportDialogUi.comboBoxImageCompression->currentIndex()).toInt());
-	bool zipCompression = compression == PDFSettings::CompressZip;
-	m_pdfExportDialogUi.spinBoxCompressionQuality->setEnabled(!zipCompression);
-	m_pdfExportDialogUi.labelCompressionQuality->setEnabled(!zipCompression);
+	bool jpegCompression = compression == PDFSettings::CompressJpeg;
+	m_pdfExportDialogUi.spinBoxCompressionQuality->setEnabled(jpegCompression);
+	m_pdfExportDialogUi.labelCompressionQuality->setEnabled(jpegCompression);
 }
 
 OutputEditorHOCR::ReadSessionData* OutputEditorHOCR::initRead(tesseract::TessBaseAPI &tess) {
@@ -421,10 +465,10 @@ void OutputEditorHOCR::addPage(QDomElement pageDiv, const QString& filename, int
 	pageItem->setData(0, ClassRole, "ocr_page");
 	pageItem->setIcon(0, QIcon(":/icons/item_page"));
 	pageItem->setCheckState(0, Qt::Checked);
-	ui.treeWidgetItems->addTopLevelItem(pageItem);
+	m_rootItem->addChild(pageItem);
 	QMap<QString,QString> langCache;
 
-	QList<QPair<QDomElement,QRect>> graphicElements;
+	QVector<QPair<QDomElement,QRect>> graphicElements;
 	QDomElement element = pageDiv.firstChildElement("div");
 	while(!element.isNull()) {
 		// Boxes without text are images
@@ -489,6 +533,15 @@ void OutputEditorHOCR::expandChildren(QTreeWidgetItem* item) const {
 	}
 }
 
+void OutputEditorHOCR::collapseChildren(QTreeWidgetItem* item) const {
+	if(item->childCount() > 0) {
+		item->setExpanded(false);
+		for(int i = 0, n = item->childCount(); i < n; ++i) {
+			collapseChildren(item->child(i));
+		}
+	}
+}
+
 bool OutputEditorHOCR::addChildItems(QDomElement element, QTreeWidgetItem* parentItem, QMap<QString,QString>& langCache) {
 	bool haveWord = false;
 	while(!element.isNull()) {
@@ -587,13 +640,13 @@ void OutputEditorHOCR::showItemProperties(QTreeWidgetItem* item) {
 	m_tool->clearSelection();
 	m_currentPageItem = nullptr;
 	m_currentItem = item;
-	if(!item) {
+	if(!item || item == m_rootItem) {
 		m_currentDocument = QDomDocument();
 		m_currentElement = QDomElement();
 		return;
 	}
 	m_currentPageItem = item;
-	while(m_currentPageItem->parent()) {
+	while(m_currentPageItem->parent() != m_rootItem) {
 		m_currentPageItem = m_currentPageItem->parent();
 	}
 	QString id = item->data(0, IdRole).toString();
@@ -643,7 +696,7 @@ void OutputEditorHOCR::showItemProperties(QTreeWidgetItem* item) {
 	}
 }
 
-bool OutputEditorHOCR::setCurrentSource(const QDomElement& pageElement, int* pageDpi) const {
+bool OutputEditorHOCR::setCurrentSource(const QDomElement& pageElement, int* pageDpi, int* overrideDpi) const {
 	if(s_pageTitleRx.indexIn(pageElement.attribute("title")) != -1) {
 		QString filename = s_pageTitleRx.cap(1);
 		int page = s_pageTitleRx.cap(2).toInt();
@@ -652,6 +705,9 @@ bool OutputEditorHOCR::setCurrentSource(const QDomElement& pageElement, int* pag
 		if(pageDpi) {
 			*pageDpi = res;
 		}
+		if(overrideDpi) {
+			res = *overrideDpi;
+		}
 
 		MAIN->getSourceManager()->addSource(filename);
 		QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
@@ -905,58 +961,79 @@ void OutputEditorHOCR::showTreeWidgetContextMenu(const QPoint &point) {
 		return;
 	}
 	QString itemClass = item->data(0, ClassRole).toString();
-	if(itemClass.isEmpty()) {
+	if(itemClass.isEmpty() && item != m_rootItem) {
 		return;
 	}
+	QMenu menu;
+	QAction* actionAddGraphic = nullptr;
+	QAction* addWordAction = nullptr;
+	QAction* ignoreWordAction = nullptr;
+	QList<QAction*> setTextActions;
+	QAction* actionRemoveItem = nullptr;
+	QAction* actionRemovePage = nullptr;
+	QAction* actionExpand = nullptr;
+	QAction* actionCollapse = nullptr;
 	if(itemClass == "ocr_page") {
-		QMenu menu;
-		QAction* actionAddGraphic = menu.addAction(_("Add graphic region"));
-		menu.addSeparator();
-		QAction* actionRemove = menu.addAction(_("Remove"));
-		QAction* clickedAction = menu.exec(ui.treeWidgetItems->mapToGlobal(point));
-		if(clickedAction == actionRemove) {
-			delete item;
-			ui.actionOutputSaveHOCR->setEnabled(ui.treeWidgetItems->topLevelItemCount() > 0);
-			ui.actionOutputExportPDF->setEnabled(ui.treeWidgetItems->topLevelItemCount() > 0);
-		} else if(clickedAction == actionAddGraphic) {
-			m_tool->clearSelection();
-			m_tool->activateDrawSelection();
+		actionAddGraphic = menu.addAction(_("Add graphic region"));
+	}
+	if(itemClass == "ocrx_word") {
+		QString prefix, suffix, trimmedWord = trimWord(item->text(0), &prefix, &suffix);
+		for(const QString& suggestion : m_spell.getSpellingSuggestions(trimmedWord)) {
+			setTextActions.append(menu.addAction(prefix + suggestion + suffix));
 		}
-	} else {
-		QMenu menu;
-		QAction* addAction = nullptr;
-		QAction* ignoreAction = nullptr;
-		if(itemClass == "ocrx_word") {
-			QString prefix, suffix, trimmedWord = trimWord(item->text(0), &prefix, &suffix);
-			for(const QString& suggestion : m_spell.getSpellingSuggestions(trimmedWord)) {
-				menu.addAction(prefix + suggestion + suffix);
-			}
-			if(menu.actions().isEmpty()) {
-				menu.addAction(_("No suggestions"))->setEnabled(false);
-			}
-			if(!m_spell.checkWord(trimWord(item->text(0)))) {
-				menu.addSeparator();
-				addAction = menu.addAction(_("Add to dictionary"));
-				ignoreAction = menu.addAction(_("Ignore word"));
-			}
+		if(setTextActions.isEmpty()) {
+			menu.addAction(_("No suggestions"))->setEnabled(false);
+		}
+		if(!m_spell.checkWord(trimWord(item->text(0)))) {
+			menu.addSeparator();
+			addWordAction = menu.addAction(_("Add to dictionary"));
+			ignoreWordAction = menu.addAction(_("Ignore word"));
+		}
+	}
+	if(item != m_rootItem) {
+		if(!menu.actions().isEmpty()) {
 			menu.addSeparator();
 		}
-		QAction* removeAction = menu.addAction(_("Remove"));
-		QAction* clickedAction = menu.exec(ui.treeWidgetItems->mapToGlobal(point));
-		if(clickedAction) {
-			if(clickedAction == addAction) {
-				m_spell.addWordToDictionary(item->text(0));
-				item->setForeground(0, item->parent()->foreground(0));
-			} else if(clickedAction == ignoreAction) {
-				m_spell.ignoreWord(item->text(0));
-				item->setForeground(0, item->parent()->foreground(0));
-			} else if(clickedAction == removeAction) {
-				removeCurrentItem();
-			} else {
-				item->setText(0, clickedAction->text());
-			}
+		if(itemClass == "ocr_page") {
+			actionRemovePage = menu.addAction(_("Remove"));
+		} else {
+			actionRemoveItem = menu.addAction(_("Remove"));
 		}
 	}
+	if(itemClass != "ocrx_word" && itemClass != "ocr_graphic") {
+		if(!menu.actions().isEmpty()) {
+			menu.addSeparator();
+		}
+		actionExpand = menu.addAction(_("Expand all"));
+		actionCollapse = menu.addAction(_("Collapse all"));
+	}
+
+	QAction* clickedAction = menu.exec(ui.treeWidgetItems->mapToGlobal(point));
+	if(!clickedAction) {
+		return;
+	}
+	if(clickedAction == actionAddGraphic) {
+		m_tool->clearSelection();
+		m_tool->activateDrawSelection();
+	} else if(clickedAction == addWordAction) {
+		m_spell.addWordToDictionary(item->text(0));
+		item->setForeground(0, item->parent()->foreground(0));
+	} else if(clickedAction == ignoreWordAction) {
+		m_spell.ignoreWord(item->text(0));
+		item->setForeground(0, item->parent()->foreground(0));
+	} else if(setTextActions.contains(clickedAction)){
+		item->setText(0, clickedAction->text());
+	} else if(clickedAction == actionRemoveItem) {
+		removeCurrentItem();
+	} else if(clickedAction == actionRemovePage) {
+		delete item;
+		ui.actionOutputSaveHOCR->setEnabled(m_rootItem->childCount() > 0);
+		ui.actionOutputExportPDF->setEnabled(m_rootItem->childCount() > 0);
+	} else if(clickedAction == actionExpand) {
+		expandChildren(item);
+	} else if(clickedAction == actionCollapse) {
+		collapseChildren(item);
+	}
 }
 
 void OutputEditorHOCR::updateFontButton(const QFont& font) {
@@ -1022,8 +1099,8 @@ bool OutputEditorHOCR::save(const QString& filename) {
 	                     "  </head>\n"
 	                     "<body>\n").arg(tess.Version());
 	file.write(header.toUtf8());
-	for(int i = 0, n = ui.treeWidgetItems->topLevelItemCount(); i < n; ++i) {
-		file.write(ui.treeWidgetItems->topLevelItem(i)->data(0, SourceRole).toString().toUtf8());
+	for(int i = 0, n = m_rootItem->childCount(); i < n; ++i) {
+		file.write(m_rootItem->child(i)->data(0, SourceRole).toString().toUtf8());
 	}
 	file.write("</body>\n</html>\n");
 	m_modified = false;
@@ -1096,6 +1173,7 @@ void OutputEditorHOCR::savePDF() {
 
 	PDFSettings pdfSettings;
 	pdfSettings.colorFormat = static_cast<QImage::Format>(m_pdfExportDialogUi.comboBoxImageFormat->itemData(m_pdfExportDialogUi.comboBoxImageFormat->currentIndex()).toInt());
+	pdfSettings.conversionFlags = pdfSettings.colorFormat == QImage::Format_Mono ? static_cast<Qt::ImageConversionFlags>(m_pdfExportDialogUi.comboBoxDithering->itemData(m_pdfExportDialogUi.comboBoxDithering->currentIndex()).toInt()) : Qt::AutoColor;
 	pdfSettings.compression = static_cast<PDFSettings::Compression>(m_pdfExportDialogUi.comboBoxImageCompression->itemData(m_pdfExportDialogUi.comboBoxImageCompression->currentIndex()).toInt());
 	pdfSettings.compressionQuality = m_pdfExportDialogUi.spinBoxCompressionQuality->value();
 	pdfSettings.useDetectedFontSizes = m_pdfExportDialogUi.checkBoxFontSize->isChecked();
@@ -1104,28 +1182,31 @@ void OutputEditorHOCR::savePDF() {
 	pdfSettings.overlay = m_pdfExportDialogUi.comboBoxOutputMode->currentIndex() == 1;
 	pdfSettings.detectedFontScaling = m_pdfExportDialogUi.spinFontScaling->value() / 100.;
 	QStringList failed;
-	for(int i = 0, n = ui.treeWidgetItems->topLevelItemCount(); i < n; ++i) {
-		QTreeWidgetItem* item = ui.treeWidgetItems->topLevelItem(i);
+	for(int i = 0, n = m_rootItem->childCount(); i < n; ++i) {
+		QTreeWidgetItem* item = m_rootItem->child(i);
 		if(item->checkState(0) != Qt::Checked) {
 			continue;
 		}
 		QRect bbox = item->data(0, BBoxRole).toRect();
 		QDomDocument doc;
 		doc.setContent(item->data(0, SourceRole).toString());
-		int pageDpi = 72;
-		if(setCurrentSource(doc.firstChildElement("div"), &pageDpi)) {
-			double dpiScale = 72. / pageDpi;
-			double imageScale = m_pdfExportDialogUi.spinBoxDpi->value() / double(pageDpi);
-			PoDoFo::PdfPage* page = document->CreatePage(PoDoFo::PdfRect(0, 0, bbox.width() * dpiScale, bbox.height() * dpiScale));
+		int sourceDpi = -1;
+		int outputDpi = m_pdfExportDialogUi.spinBoxDpi->value();
+		if(setCurrentSource(doc.firstChildElement("div"), &sourceDpi, &outputDpi)) {
+			double docScale = (72. / sourceDpi);
+			double imgScale = double(outputDpi) / sourceDpi;
+			PoDoFo::PdfPage* page = document->CreatePage(PoDoFo::PdfRect(0, 0, bbox.width() * docScale, bbox.height() * docScale));
 			painter.SetPage(page);
 			painter.SetFont(font);
 
-			PoDoFoPDFPainter pdfprinter(document, &painter, dpiScale, imageScale);
+			PoDoFoPDFPainter pdfprinter(document, &painter, docScale);
 			pdfprinter.setFontSize(m_pdfFontDialog.currentFont().pointSize());
-			printChildren(pdfprinter, item, pdfSettings);
+			printChildren(pdfprinter, item, pdfSettings, imgScale);
 			if(pdfSettings.overlay) {
-				pdfprinter.drawImage(bbox, m_tool->getSelection(bbox), pdfSettings);
+				QRect scaledBBox(imgScale * bbox.left(), imgScale * bbox.top(), imgScale * bbox.width(), imgScale * bbox.height());
+				pdfprinter.drawImage(bbox, m_tool->getSelection(scaledBBox), pdfSettings);
 			}
+			MAIN->getDisplayer()->setResolution(sourceDpi);
 			painter.FinishPage();
 		} else {
 			failed.append(item->text(0));
@@ -1138,7 +1219,7 @@ void OutputEditorHOCR::savePDF() {
 	delete document;
 }
 
-void OutputEditorHOCR::printChildren(PDFPainter& painter, QTreeWidgetItem* item, const PDFSettings& pdfSettings) const {
+void OutputEditorHOCR::printChildren(PDFPainter& painter, QTreeWidgetItem* item, const PDFSettings& pdfSettings, double imgScale) const {
 	if(item->checkState(0) != Qt::Checked) {
 		return;
 	}
@@ -1181,10 +1262,11 @@ void OutputEditorHOCR::printChildren(PDFPainter& painter, QTreeWidgetItem* item,
 			painter.drawText(wordRect.x(), y, wordItem->text(0));
 		}
 	} else if(itemClass == "ocr_graphic" && !pdfSettings.overlay) {
-		painter.drawImage(itemRect, m_tool->getSelection(itemRect), pdfSettings);
+		QRect scaledItemRect(itemRect.left() * imgScale, itemRect.top() * imgScale, itemRect.width() * imgScale, itemRect.height() * imgScale);
+		painter.drawImage(itemRect, m_tool->getSelection(scaledItemRect), pdfSettings);
 	} else {
 		for(int i = 0, n = item->childCount(); i < n; ++i) {
-			printChildren(painter, item->child(i), pdfSettings);
+			printChildren(painter, item->child(i), pdfSettings, imgScale);
 		}
 	}
 }
@@ -1194,12 +1276,12 @@ void OutputEditorHOCR::updatePreview() {
 		return;
 	}
 	m_preview->setVisible(m_pdfExportDialogUi.checkBoxPreview->isChecked());
-	if(ui.treeWidgetItems->topLevelItemCount() == 0 || !m_pdfExportDialogUi.checkBoxPreview->isChecked()) {
+	if(m_rootItem->childCount() == 0 || !m_pdfExportDialogUi.checkBoxPreview->isChecked()) {
 		return;
 	}
 	QTreeWidgetItem* item = ui.treeWidgetItems->currentItem();
 	if(!item) {
-		item = ui.treeWidgetItems->topLevelItem(0);
+		item = m_rootItem->child(0);
 	} else {
 		while(item->parent()) {
 			item = item->parent();
@@ -1207,12 +1289,13 @@ void OutputEditorHOCR::updatePreview() {
 	}
 	QRect bbox = item->data(0, BBoxRole).toRect();
 	QDomDocument doc;
-	int pageDpi = 72;
+	int pageDpi = -1;
 	doc.setContent(item->data(0, SourceRole).toString());
 	setCurrentSource(doc.firstChildElement("div"), &pageDpi);
 
 	PDFSettings pdfSettings;
 	pdfSettings.colorFormat = static_cast<QImage::Format>(m_pdfExportDialogUi.comboBoxImageFormat->itemData(m_pdfExportDialogUi.comboBoxImageFormat->currentIndex()).toInt());
+	pdfSettings.conversionFlags = pdfSettings.colorFormat == QImage::Format_Mono ? static_cast<Qt::ImageConversionFlags>(m_pdfExportDialogUi.comboBoxDithering->itemData(m_pdfExportDialogUi.comboBoxDithering->currentIndex()).toInt()) : Qt::AutoColor;
 	pdfSettings.compression = static_cast<PDFSettings::Compression>(m_pdfExportDialogUi.comboBoxImageCompression->itemData(m_pdfExportDialogUi.comboBoxImageCompression->currentIndex()).toInt());
 	pdfSettings.compressionQuality = m_pdfExportDialogUi.spinBoxCompressionQuality->value();
 	pdfSettings.useDetectedFontSizes = m_pdfExportDialogUi.checkBoxFontSize->isChecked();
@@ -1255,11 +1338,15 @@ bool OutputEditorHOCR::clear(bool hide) {
 		}
 	}
 	m_idCounter = 0;
-	ui.treeWidgetItems->clear();
+	while(m_rootItem->childCount() > 0) {
+		delete m_rootItem->child(0);
+	}
 	ui.tableWidgetProperties->setRowCount(0);
 	ui.plainTextEditOutput->clear();
 	m_tool->clearSelection();
 	m_modified = false;
+	ui.actionOutputSaveHOCR->setEnabled(false);
+	ui.actionOutputExportPDF->setEnabled(false);
 	if(hide)
 		MAIN->setOutputPaneVisible(false);
 	return true;
