@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * OutputEditorText.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -136,8 +136,8 @@ void OutputEditorText::filterBuffer() {
 			preChars += "\\.\\?!"; // Keep if preceded by end mark (.?!)
 		}
 		if(ui.actionOutputPostprocKeepQuote->isChecked()) {
-			preChars += "'\""; // Keep if preceded by dot
-			sucChars += "'\""; // Keep if succeeded by dot
+			preChars += "'\"\u00BB\u00AB"; // Keep if preceded by quote
+			sucChars += "'\"\u00AB\u00BB"; // Keep if succeeded by quote
 		}
 		if(ui.actionOutputPostprocKeepParagraphs->isChecked()) {
 			sucChars += "\u2029"; // Keep if succeeded by line break
@@ -194,10 +194,23 @@ void OutputEditorText::findReplace(bool backwards, bool replace) {
 }
 
 void OutputEditorText::read(tesseract::TessBaseAPI &tess, ReadSessionData *data) {
-	char* text = tess.GetUTF8Text();
+	char* textbuf = tess.GetUTF8Text();
+	QString text = QString::fromUtf8(textbuf);
+	if(!text.endsWith('\n'))
+		text.append('\n');
+	if(data->prependFile || data->prependPage) {
+		QStringList prepend;
+		if(data->prependFile) {
+            prepend.append(_("File: %1").arg(data->file));
+		}
+		if(data->prependPage) {
+            prepend.append(_("Page: %1").arg(data->page));
+		}
+		text.prepend(QString("[%1]\n").arg(prepend.join("; ")));
+	}
 	bool& insertText = static_cast<TextReadSessionData*>(data)->insertText;
-	QMetaObject::invokeMethod(this, "addText", Qt::QueuedConnection, Q_ARG(QString, QString::fromUtf8(text)), Q_ARG(bool, insertText));
-	delete[] text;
+	QMetaObject::invokeMethod(this, "addText", Qt::QueuedConnection, Q_ARG(QString, text), Q_ARG(bool, insertText));
+	delete[] textbuf;
 	insertText = true;
 }
 
