@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Recognizer.hh
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -46,7 +46,6 @@
 #include "Recognizer.hh"
 #include "TessdataManager.hh"
 #include "Utils.hh"
-#include "ui_PageRangeDialog.h"
 
 struct Recognizer::ProgressMonitor : public MainWindow::ProgressMonitor {
 	ETEXT_DESC desc;
@@ -85,11 +84,7 @@ Recognizer::Recognizer(const UI_MainWindow& _ui) :
 	m_menuPages->addAction(multiplePagesAction);
 
 	m_pagesDialog = new QDialog(MAIN);
-	Ui::PageRangeDialog uiPageRangeDialog;
-	uiPageRangeDialog.setupUi(m_pagesDialog);
-	m_pagesLineEdit = uiPageRangeDialog.lineEditPageRange;
-	m_pageAreaLabel = uiPageRangeDialog.labelRecognitionArea;
-	m_pageAreaComboBox = uiPageRangeDialog.comboBoxRecognitionArea;
+	m_pagesDialogUi.setupUi(m_pagesDialog);
 
 	ui.toolButtonRecognize->setText(QString("%1\n%2").arg(m_modeLabel).arg(m_langLabel));
 	ui.menuLanguages->installEventFilter(this);
@@ -97,11 +92,13 @@ Recognizer::Recognizer(const UI_MainWindow& _ui) :
 	connect(ui.toolButtonRecognize, SIGNAL(clicked()), this, SLOT(recognizeButtonClicked()));
 	connect(currentPageAction, SIGNAL(triggered()), this, SLOT(recognizeCurrentPage()));
 	connect(multiplePagesAction, SIGNAL(triggered()), this, SLOT(recognizeMultiplePages()));
-	connect(uiPageRangeDialog.lineEditPageRange, SIGNAL(textChanged(QString)), this, SLOT(clearLineEditPageRangeStyle()));
+	connect(m_pagesDialogUi.lineEditPageRange, SIGNAL(textChanged(QString)), this, SLOT(clearLineEditPageRangeStyle()));
 
 	MAIN->getConfig()->addSetting(new VarSetting<QString>("language", "eng:en_EN"));
-	MAIN->getConfig()->addSetting(new ComboSetting("ocrregionstrategy", uiPageRangeDialog.comboBoxRecognitionArea, 0));
-	MAIN->getConfig()->addSetting(new VarSetting<bool>("osd", false));
+	MAIN->getConfig()->addSetting(new ComboSetting("ocrregionstrategy", m_pagesDialogUi.comboBoxRecognitionArea, 0));
+	MAIN->getConfig()->addSetting(new SwitchSetting("ocraddsourcefilename", m_pagesDialogUi.checkBoxPrependFilename));
+	MAIN->getConfig()->addSetting(new SwitchSetting("ocraddsourcepage", m_pagesDialogUi.checkBoxPrependPage));
+	MAIN->getConfig()->addSetting(new VarSetting<int>("psm", 6));
 }
 
 QStringList Recognizer::getAvailableLanguages() const {
@@ -174,7 +171,9 @@ void Recognizer::updateLanguagesMenu() {
 	delete m_langMenuCheckGroup;
 	m_langMenuCheckGroup = new QActionGroup(this);
 	m_langMenuCheckGroup->setExclusive(false);
-	m_osdAction = nullptr;
+	delete m_psmCheckGroup;
+	m_psmCheckGroup = new QActionGroup(this);
+	connect(m_psmCheckGroup, SIGNAL(triggered(QAction*)), this, SLOT(psmSelected(QAction*)));
 	m_menuMultilanguage = nullptr;
 	m_curLang = Config::Lang();
 	QAction* curitem = nullptr;
@@ -282,16 +281,41 @@ void Recognizer::updateLanguagesMenu() {
 	if(activeitem)
 		activeitem->trigger();
 
-	// Add OSD item
-	if(haveOsd) {
-		ui.menuLanguages->addSeparator();
-		m_osdAction = new QAction(_("Detect script and orientation"), ui.menuLanguages);
-		m_osdAction->setCheckable(true);
-		m_osdAction->setChecked(MAIN->getConfig()->getSetting<VarSetting<bool>>("osd")->getValue());
-		connect(m_osdAction, SIGNAL(toggled(bool)), this, SLOT(osdToggled(bool)));
-		ui.menuLanguages->addAction(m_osdAction);
+	// Add PSM items
+	ui.menuLanguages->addSeparator();
+	QMenu* psmMenu = new QMenu();
+	int activePsm = MAIN->getConfig()->getSetting<VarSetting<int>>("psm")->getValue();
+
+	struct PsmEntry {
+		QString label;
+		tesseract::PageSegMode psmMode;
+		bool requireOsd;
+	};
+	QVector<PsmEntry> psmModes = {
+			PsmEntry{_("Automatic page segmentation"), tesseract::PSM_AUTO, false},
+			PsmEntry{_("Page segmentation with orientation and script detection"), tesseract::PSM_AUTO_OSD, true},
+			PsmEntry{_("Assume single column of text"), tesseract::PSM_SINGLE_COLUMN, false},
+			PsmEntry{_("Assume single block of vertically aligned text"), tesseract::PSM_SINGLE_BLOCK_VERT_TEXT, false},
+			PsmEntry{_("Assume a single uniform block of text"), tesseract::PSM_SINGLE_BLOCK, false},
+			PsmEntry{_("Assume a line of text"), tesseract::PSM_SINGLE_LINE, false},
+			PsmEntry{_("Assume a single word"), tesseract::PSM_SINGLE_WORD, false},
+			PsmEntry{_("Assume a single word in a circle"), tesseract::PSM_CIRCLE_WORD, false},
+			PsmEntry{_("Sparse text in no particular order"), tesseract::PSM_SPARSE_TEXT, false},
+			PsmEntry{_("Sparse text with orientation and script detection"), tesseract::PSM_SPARSE_TEXT_OSD, true}};
+	for(const auto& entry : psmModes) {
+		QAction* item = psmMenu->addAction(entry.label);
+		item->setData(entry.psmMode);
+		item->setEnabled(!entry.requireOsd || haveOsd);
+		item->setCheckable(true);
+		item->setChecked(activePsm == entry.psmMode);
+		m_psmCheckGroup->addAction(item);
 	}
 
+	QAction* psmAction = new QAction(_("Page segmentation mode"), ui.menuLanguages);
+	psmAction->setMenu(psmMenu);
+	ui.menuLanguages->addAction(psmAction);
+
+
 	// Add installer item
 	ui.menuLanguages->addSeparator();
 	ui.menuLanguages->addAction(_("Manage languages..."), this, SLOT(manageInstalledLanguages()));
@@ -318,7 +342,7 @@ void Recognizer::setMultiLanguage() {
 	QString langs;
 	for(QAction* action : m_langMenuCheckGroup->actions()) {
 		if(action->isChecked()) {
-			langs += action->data().value<QString>() + "+";
+			langs += action->data().toString() + "+";
 		}
 	}
 	if(langs.isEmpty()) {
@@ -341,22 +365,24 @@ void Recognizer::clearLineEditPageRangeStyle() {
 	qobject_cast<QLineEdit*>(QObject::sender())->setStyleSheet("");
 }
 
-void Recognizer::osdToggled(bool state) {
-	MAIN->getConfig()->getSetting<VarSetting<bool>>("osd")->setValue(state);
+void Recognizer::psmSelected(QAction *action) {
+	MAIN->getConfig()->getSetting<VarSetting<int>>("psm")->setValue(action->data().toInt());
 }
 
 QList<int> Recognizer::selectPages(bool& autodetectLayout) {
 	int nPages = MAIN->getDisplayer()->getNPages();
 
-	m_pagesLineEdit->setText(QString("1-%1").arg(nPages));
-	m_pagesLineEdit->setFocus();
-	m_pageAreaLabel->setVisible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
-	m_pageAreaComboBox->setVisible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
-	m_pageAreaComboBox->setItemText(0, MAIN->getDisplayer()->hasMultipleOCRAreas() ? _("Current selection") : _("Entire page"));
+	m_pagesDialogUi.lineEditPageRange->setText(QString("1-%1").arg(nPages));
+	m_pagesDialogUi.lineEditPageRange->setFocus();
+	m_pagesDialogUi.labelRecognitionArea->setVisible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
+	m_pagesDialogUi.comboBoxRecognitionArea->setVisible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
+	m_pagesDialogUi.groupBoxPrepend->setVisible(MAIN->getDisplayer()->allowAutodetectOCRAreas());
+
+	m_pagesDialogUi.comboBoxRecognitionArea->setItemText(0, MAIN->getDisplayer()->hasMultipleOCRAreas() ? _("Current selection") : _("Entire page"));
 
 	QList<int> pages;
 	if(m_pagesDialog->exec() == QDialog::Accepted) {
-		QString text = m_pagesLineEdit->text();
+		QString text = m_pagesDialogUi.lineEditPageRange->text();
 		text.replace(QRegExp("\\s+"), "");
 		for(const QString& block : text.split(',', QString::SkipEmptyParts)) {
 			QStringList ranges = block.split('-', QString::SkipEmptyParts);
@@ -377,11 +403,11 @@ QList<int> Recognizer::selectPages(bool& autodetectLayout) {
 			}
 		}
 		if(pages.empty()) {
-			m_pagesLineEdit->setStyleSheet("background: #FF7777; color: #FFFFFF;");
+			m_pagesDialogUi.lineEditPageRange->setStyleSheet("background: #FF7777; color: #FFFFFF;");
 		}
 	}
 	qSort(pages);
-	autodetectLayout = m_pageAreaComboBox->isVisible() ? m_pageAreaComboBox->currentIndex() == 1 : false;
+	autodetectLayout = m_pagesDialogUi.comboBoxRecognitionArea->isVisible() ? m_pagesDialogUi.comboBoxRecognitionArea->currentIndex() == 1 : false;
 	return pages;
 }
 
@@ -410,17 +436,18 @@ void Recognizer::recognizeMultiplePages() {
 
 void Recognizer::recognize(const QList<int> &pages, bool autodetectLayout) {
 	tesseract::TessBaseAPI tess;
+	bool prependFile = pages.size() > 1 && MAIN->getConfig()->getSetting<SwitchSetting>("ocraddsourcefilename")->getValue();
+	bool prependPage = pages.size() > 1 && MAIN->getConfig()->getSetting<SwitchSetting>("ocraddsourcepage")->getValue();
 	if(initTesseract(tess, m_curLang.prefix.toLocal8Bit().constData())) {
 		QString failed;
-		if(MAIN->getConfig()->getSetting<VarSetting<bool>>("osd")->getValue() == true) {
-			tess.SetPageSegMode(tesseract::PSM_AUTO_OSD);
-		}
+		tess.SetPageSegMode(static_cast<tesseract::PageSegMode>(m_psmCheckGroup->checkedAction()->data().toInt()));
 		OutputEditor::ReadSessionData* readSessionData = MAIN->getOutputEditor()->initRead(tess);
 		ProgressMonitor monitor(pages.size());
 		MAIN->showProgress(&monitor);
 		Utils::busyTask([&] {
 			int npages = pages.size();
 			int idx = 0;
+			QString prevFile;
 			for(int page : pages) {
 				monitor.desc.progress = 0;
 				++idx;
@@ -436,7 +463,14 @@ void Recognizer::recognize(const QList<int> &pages, bool autodetectLayout) {
 				readSessionData->file = MAIN->getDisplayer()->getCurrentImage(readSessionData->page);
 				readSessionData->angle = MAIN->getDisplayer()->getCurrentAngle();
 				readSessionData->resolution = MAIN->getDisplayer()->getCurrentResolution();
+				bool firstChunk = true;
+				bool newFile = readSessionData->file != prevFile;
+				prevFile = readSessionData->file;
 				for(const QImage& image : MAIN->getDisplayer()->getOCRAreas()) {
+					readSessionData->prependPage = prependPage && firstChunk;
+					readSessionData->prependFile = prependFile && (readSessionData->prependPage || newFile);
+					firstChunk = false;
+					newFile = false;
 					tess.SetImage(image.bits(), image.width(), image.height(), 4, image.bytesPerLine());
 					tess.SetSourceResolution(MAIN->getDisplayer()->getCurrentResolution());
 					tess.Recognize(&monitor.desc);
