@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * TessdataManager.hh
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -34,6 +34,7 @@
 #include <QMessageBox>
 #include <QUrl>
 #include <QVBoxLayout>
+#include <tesseract/baseapi.h>
 
 #include "Config.hh"
 #include "MainWindow.hh"
@@ -85,14 +86,17 @@ bool TessdataManager::setup() {
 }
 
 bool TessdataManager::fetchLanguageList(QString& messages) {
-	QByteArray data = Utils::download(QUrl("https://api.github.com/repos/tesseract-ocr/tessdata/contents"), messages);
+	m_languageList->clear();
 
+	// Get newest tag older or equal to used tesseract version
+	QByteArray data = Utils::download(QUrl("https://api.github.com/repos/tesseract-ocr/tessdata/tags"), messages);
 	if(data.isEmpty()) {
 		messages = _("Failed to fetch list of available languages: %1").arg(messages);
 		return false;
 	}
 
-	QList<QPair<QString,QString>> extraFiles;
+	QString tessdataVer;
+	QString tessVer(TESSERACT_VERSION_STR);
 #if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 	QJson::Parser parser;
 	bool ok = false;
@@ -102,9 +106,7 @@ bool TessdataManager::fetchLanguageList(QString& messages) {
 		return false;
 	}
 	for(const QVariant& value : json) {
-		QVariantMap treeObj = value.toMap();
-		QString name = treeObj.value("name").toString();
-		QString url = treeObj.value("download_url").toString();
+		QString tag = value.toMap().value("name").toString();
 #else
 	QJsonParseError err;
 	QJsonDocument json = QJsonDocument::fromJson(data, &err);
@@ -112,10 +114,43 @@ bool TessdataManager::fetchLanguageList(QString& messages) {
 		messages = _("Parsing error: %1").arg(err.errorString());
 		return false;
 	}
+	for(const QJsonValue& value : json.array()) {
+		QString tag = value.toObject().value("name").toString();
+#endif
+		if(tag <= tessVer && tag > tessdataVer) {
+			tessdataVer = tag;
+		}
+	}
+	data = Utils::download(QUrl("https://api.github.com/repos/tesseract-ocr/tessdata/contents?ref=" + tessdataVer), messages);
+
+	if(data.isEmpty()) {
+		messages = _("Failed to fetch list of available languages: %1").arg(messages);
+		return false;
+	}
+
+	QVector<QPair<QString,QString>> extraFiles;
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+	ok = false;
+	json = parser.parse( data, &ok ).toList();
+	if(!ok) {
+		messages = _("Parsing error: %1").arg(parser.errorLine() + ": " + parser.errorString());
+		return false;
+	}
+	for(const QVariant& value : json) {
+		QVariantMap treeObj = value.toMap();
+		QString name = treeObj.value("name").toString();
+		QString url = treeObj.value("download_url").toString();
+#else
+	err = QJsonParseError();
+	json = QJsonDocument::fromJson(data, &err);
+	if(json.isNull()) {
+		messages = _("Parsing error: %1").arg(err.errorString());
+		return false;
+	}
 	for(const QJsonValue& value : json.array()) {
 		QJsonObject treeObj = value.toObject();
-		QString name = treeObj.find("name").value().toString();
-		QString url = treeObj.find("download_url").value().toString();
+		QString name = treeObj.value("name").toString();
+		QString url = treeObj.value("download_url").toString();
 #endif
 		if(name.endsWith(".traineddata")) {
 			m_languageFiles[name.left(name.indexOf("."))].append({name,url});
