@@ -1,7 +1,7 @@
 /* -*- Mode: C++; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
 /*
  * Utils.cc
- * Copyright (C) 2013-2016 Sandro Mani <manisandro@gmail.com>
+ * Copyright (C) 2013-2017 Sandro Mani <manisandro@gmail.com>
  *
  * gImageReader is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -127,7 +127,7 @@ bool Utils::handleSourceDragEvent(const QMimeData* mimeData) {
 	}
 	QList<QByteArray> formats = QImageReader::supportedImageFormats();
 	formats.append("pdf");
-	for(const QUrl url : mimeData->urls()) {
+	for(const QUrl& url : mimeData->urls()) {
 		QFile file(url.toLocalFile());
 		if(!file.exists()) {
 			continue;
@@ -149,7 +149,7 @@ void Utils::handleSourceDropEvent(const QMimeData* mimeData) {
 	}
 	QList<QByteArray> formats = QImageReader::supportedImageFormats();
 	formats.append("pdf");
-	for(const QUrl url : mimeData->urls()) {
+	for(const QUrl& url : mimeData->urls()) {
 		QFile file(url.toLocalFile());
 		if(!file.exists()) {
 			continue;
